#coding: utf-8
import json
import os
import uuid
import logging
import traceback

from django.conf import settings
from django.template.base import Template
from django.template.context import RequestContext
from django.http import HttpResponse
import django.utils.importlib as importlib

from lxml import etree

logger = logging.getLogger("critical")

from django.conf import settings

settings_arguments = ['MEDIA_ROOT',
                      'BLANK_EDITOR_MEDIA_URL',
                      'BLANK_EDITOR_STATIC_URL',
                      'BLANK_EDITOR_MEDIA_ROOT',
                      'BLANK_EDITOR_STATIC_ROOT']


for arg in settings_arguments:
    if not getattr(settings, arg, None):
        raise Exception(u"Provide settings argument - %s" % (arg, ))

strict_file_paths = [
    os.path.join(settings.BLANK_EDITOR_MEDIA_ROOT, 'downloads'),
    os.path.join(settings.BLANK_EDITOR_MEDIA_ROOT, 'reports'),
    settings.BLANK_EDITOR_CONFIG_DIR,
    settings.BLANK_EDITOR_TEMPLATE_DIR
]

for path in strict_file_paths:
    if not os.path.exists(path):
        os.mkdir(path)


class BlankGenerator(object):

    _download_dir = strict_file_paths[0]
    _reports_dir  = strict_file_paths[1]
    _config_dir   = strict_file_paths[2]
    _template_dir = strict_file_paths[3]
    default_blank_name = "default_blank.html"
    class NonUniqueProviderName(Exception):
        pass
    class ProviderMethodsInconsistency(Exception):
        pass

    def __search_config(self):

        if not getattr(settings, "BLANK_EDITOR_CONFIG_DIR", False):
            search_path = os.path.join(
                os.path.normpath(os.path.dirname(__file__)), u"config")
        else:
            search_path = getattr(settings, "BLANK_EDITOR_CONFIG_DIR")

        if not os.path.exists(search_path):
            os.mkdir(search_path)

        return search_path

    @staticmethod
    def get_url_config():
        return {
            "get_templates":        "load_templates",
            "save_new_template":    "save_template",
            "save_new_background":  "upload_blank",
            "get_report_data":      "get_data_for_report",
            "start_editor":         "print_special"
        }

    def __install_providers(self):
        """достаем конфигурационный файл
        """
        config_path = self.__search_config()
        with open(os.path.join(config_path, 'config.xml'), 'rb') as config_file:
            xml_ = etree.parse(config_file)
            main_config = xml_.getroot()
            for report in main_config:
                provider_path = report.findall(".//Provider")[0]
                print(provider_path.text)
                #определяем короткое имя для поиска поставщика данных
                provider_path = provider_path.text.split(".")
                provider_name = provider_path[-1]
                if provider_name in self.__providers:
                    raise self.NonUniqueProviderName()
                provider_module = importlib.import_module(".".join(provider_path[:-1]))
                provider_class = getattr(provider_module, provider_name)
                provider_instance = provider_class()
                provider_instance.name = provider_name
                provider_instance.title = dict(report.attrib)['title']
                provider_instance.template_name = report.findall(".//TemplateName")[0].text
                provider_instance.num_of_fields = report.findall(".//NumOfFields")[0].text
                provider_instance.blank_size = report.findall(".//BlankSize")[0].text
                check_attributes = ['getDataForReport']
                verified = all(map(lambda x: getattr(provider_instance, x, False), check_attributes))
                if not verified:
                    raise self.ProviderMethodsInconsistency()

                self.__providers[provider_name] = provider_instance

    def get_provider(self, name):
        print(self.__providers)
        return self.__providers.get(name, None)

    def __init__(self, provider_name, direct_id=None, get_values_elements=None, add_values=None):

        self.__providers = dict()
        self.__install_providers()

        self.default_blank_path = os.path.join(os.path.dirname(__file__), 'templates')

        if not get_values_elements:
            get_values_elements = []

        provider = self.__current_provider = self.get_provider(provider_name)
        self.template_name = provider.template_name
        self.blank_size = provider.blank_size
        self.num_of_fields = provider.num_of_fields

        self.direct_id = direct_id
        self.get_values_elements = get_values_elements
        self.add_values = add_values

        mp = self.media_path
        pe = os.path.exists(mp)
        if not pe:
            os.mkdir(mp)

        tp = self.template_path
        te = os.path.exists(tp)
        if not te:
            os.mkdir(tp)

        dd = self.download_path
        de = os.path.exists(dd)
        if not de:
            os.mkdir(dd)

    @property
    def media_path(self):
        #путь для хранения задних фонов для бланков
        dir = self._reports_dir
        ddir = os.path.join(dir, self.provider.template_name).replace("\\", '/')
        return ddir

    @property
    def template_path(self):
        #путь для хранения шаблонов
        dir = self._template_dir
        ddir = os.path.join(dir, self.provider.template_name).replace("\\", '/')
        return ddir

    @property
    def download_path(self):
        #путь для хранения загруженных файлов бланков
        dir = self._download_dir
        ddir = os.path.join(dir, self.provider.template_name).replace("\\", '/')
        return ddir

    @property
    def provider(self):
        return self.__current_provider

    def get_print_handler(self, client_id, user, **kwargs):
        """
        задействование редактора бланков для отчета.
        Добавляет на форму кнопку печати
        client_id - идентификатор компонента из которого берется номер записи журнала из которой извлекаются
            необходимые данные для отчета
        report_url - урл пака для получения доступа к метода извлечения данных из отчета
        template_name - имя шаблона для редактора бланков
        blank_size - размер бланка используемого по умолчанию
        """
        params = {}
        #определяем путь где хранится параметры бланка
        report_path = self.template_path
        params['path_template'] = os.path.join(report_path,
            self.template_name + ".html").replace("\\", '/')
        params['blank_size'] = self.blank_size

        width, height = self.blank_size.split(',')
        test = (len(self.blank_size.split(',')) == 2
                and int(width) and int(height))

        if not test:
            raise Exception(u"Неправильно задан формат для размеров бланка отчета! Формат - \"000,000\"")
        try:
            search_template_file = "template_uid{0:0>4}_".format(user.id)
            search_basic_file = "template_uid0000_"
            found_files = []
            #ищем в папке с отчетом отредактированный базовый шаблон в формате HTML
            #договариваемся что его название начинается на template
            #по умолчанию на редактирование открывается измененный по номеру документ
            #если таковой не находится то берем базовый шаблон и редактируем его (обычно он находится в папке /templates)
            search_path = report_path.replace("\\", "/")
            print(u"PRINT BLANK -> Searching saved blanks in ->%s for ->%s" % (
                search_path, search_template_file))

            for dirname, dirnames, filenames in os.walk(search_path):

                for file in filenames:
                    if search_template_file in file or search_basic_file in file:
                        #шаблон найден
                        found_files.append(file)

            #не найден сохраненный шаблон для отчета
            if not found_files:
                print(u"PRINT BLANK -> No print blank found!")
                #стандартный шаблон хранится в этой папке
                report_path = self.default_blank_path
                last_edit_file = self.default_blank_name
                params['template_name'] = self.template_name
            else:
                #подбираем последний измененный файл
                last_edit_file = reduce(lambda x, y: x if x[1] > y[1] else y,
                    [(fname, os.path.getmtime(os.path.join(search_path, fname)))
                    for fname in found_files])

                if isinstance(last_edit_file, tuple):
                    last_edit_file = last_edit_file[0]

            params['path_template'] = os.path.join(report_path, last_edit_file).replace("\\",'/')

            print(u"PRINT BLANK -> Opening files->%s" % (params['path_template']))

            f = open(params['path_template'])
            #выведем имя нового шаблона и сократим название до существенного (слово template является служебным словом)
            params['template_name'] = last_edit_file.replace("template_", '').replace(".html", '')
            handler = u"""
                var config = {
                    get_report_data_url:    '%(get_report_data_url)s',
                    start_editor_url:       '%(path)s',
                    field_id_value:         '%(field_client_id)s',
                    provider_name:          '%(provider_name)s',
                    num_of_params:          '%(num_of_pars)s',
                    model_id:               '%(model_id)s',
                    additional_values:      %(additional_values)s,
                    report_params: {
                                            %(params)s
                    }
                };
                if (modifyConfig && typeof(modifyConfig)=="function") {
                    modifyConfig(config);
                }
                startBlankEditor(config, preStartFunc, postStartFunc);
            """ % ({
                "get_report_data_url": self.get_url_config()['get_report_data'],
                "path": self.get_url_config()['start_editor'],
                "field_client_id": client_id,
                "provider_name": self.__current_provider.name,
                "params": ",\n".join(["%s:'%s'" % (k, v) for k, v in params.items() if k != "report_data"]),
                "num_of_pars": self.num_of_fields if self.num_of_fields else 0,
                "model_id": self.direct_id,
                "additional_values": ("[" + ",".join(["\"" + i + "\"" for i in self.get_values_elements]) + "]")
                if self.get_values_elements else ("{" + ",".join([(i + ": \'" + str(j) + "\'")
                for i, j in self.add_values.items()]) + "}") if self.add_values else "[]"
            })

        except IOError as e:
            logger.critical(u"%s\n\r\t%s" % (unicode(e),
                                            traceback.print_exc()))

        return handler

    def get_report_response(self, config, request, exist_file = None):

        url_config = self.get_url_config()
        get_data_url = url_config['get_report_data']
        path_template = config['path_template']
        number_of_params = config['number_of_params']
        pack_url = config['pack_url']
        id_field = config['id_field']
        blank_size = config['blank_size']
        report_data = config['report_data']
        data_description = config['data_description']
        background_reload_url = config.get("background_reload_url", '')
        response_config = {"template_name": u""}
        path_to_file = path_template.strip()
        file_name = "report_%s" % (str(uuid.uuid4())[0:6], )
        #если путь к шаблону пустой значит данный экшен вызван вторично при передаче определенных параметров
        download_folder = "/downloads/"
        download_part_url = self.download_path.split(download_folder)[1]
        media_url = settings.BLANK_EDITOR_MEDIA_URL
        media_url = "/" + media_url.strip("/") + "/"

        download_url = os.path.join(media_url, download_folder.strip("/"), download_part_url, file_name + ".html").replace('\\','/')

        if exist_file:
            temp_file_path = exist_file.strip()
        else:
            temp_file_path = os.path.join(self.download_path, file_name + ".html").replace('\\', '/')

        response_config['template_name'] = (path_to_file.split('/')[-1].
                replace("template_", '').replace(".html", '').replace("uid", "")[5:])

        try:
            kvit_file = open(path_to_file)
            html_template = Template(kvit_file.read())
            kvit_file.close()
        except IOError as e:
            logger.critical(u"%s\n\r\t%s" % (unicode(e),
                                             traceback.print_exc()))
            raise

        #TODO - здесь надо формировать словарь атриббутов для отчета
        data_tags = {
            "template_name": response_config['template_name'],
            "template_path": path_to_file,
            "tempfile_path": temp_file_path,
            "number_of_params": number_of_params,
            "get_params_url": get_data_url,
            "pack_url": pack_url,
            "work_path": settings.PROJECT_ROOT.replace('\\', '/'),
            "blank_size": blank_size,
            "id_field": id_field,
            "background_reload_url": background_reload_url,
            "load_template_url": url_config['get_templates'],
            "save_template_url": url_config['save_new_template'],
            "upload_blank": url_config['save_new_background']
        }
        #дополнительные параметры прозрачно передающиеся и нужные для
        #загрузки данных для редактора
        data_tags['additional_values'] = config.get('add_vals', '').strip()
        params_val_list = []
        params_desc_list = []
        t = 0
        report_data = json.loads(report_data) if isinstance(report_data, basestring) else report_data
        for k in report_data:
            t += 1
            params_val_list.append(k)
        data_description = json.loads(data_description) if isinstance(data_description, basestring) else data_description
        for k in data_description:
            params_desc_list.append(k)

        data_tags['report_data'] = params_val_list
        data_tags['data_description'] = params_desc_list

        prefix = u"../".join(["" for f in self.download_path.split("/")])
        data_tags["BLANK_EDITOR_STATIC_URL"] = prefix + settings.BLANK_EDITOR_STATIC_URL
        data_tags["BLANK_EDITOR_MEDIA_URL"] = prefix + settings.BLANK_EDITOR_MEDIA_URL
        html = html_template.render(RequestContext(request, data_tags))
        try:
            with open(temp_file_path, "w") as fh:
                fh.write(html.encode("utf-8"))
        except IOError as e:
            logger.critical(u"%s\n\r\t%s" % (unicode(e),
                                             traceback.print_exc()))
            raise

        if exist_file:
            generated_report = open(exist_file.strip(), 'r').read()
            template = Template(generated_report)
            rendered_template = template.render(
                RequestContext(request,
                {"BLANK_EDITOR_STATIC_URL": prefix + settings.BLANK_EDITOR_STATIC_URL,
                 "BLANK_EDITOR_MEDIA_URL": prefix + settings.BLANK_EDITOR_MEDIA_URL
            }))
            return HttpResponse(rendered_template)
        else:
            data = {"location": download_url}
            return HttpResponse(content_type="json", content=json.dumps(data))

    def get_saved_blanks_by_user(self, user):
        """
            возвращает список имен с сохраненными шаблонами
            для пользователя
        """
        data = []
        path_to_report = self.template_path
        #формат сохраненного файла шаблона выглядить следующим образом
        #template_uidXXXX_[name].html где XXXX - это id пользователя в системе
        #а name - это произвольное имя шаблона заданное пользователем
        for dirname, dirnames, filenames in os.walk(path_to_report):

            file_filter_func = lambda x: x if "template_uid{0:0>4}_".format(user.id)\
                                              in x or u"template_uid0000_basic" in x else None
            if user.is_superuser:
                file_filter_func = lambda x: x if u"template_uid" in x else None
            filt_names = [f for f in filenames]
            filt_names = filter(file_filter_func, filt_names)
            filt_names = map(lambda f:
            f.replace(".html", '').replace("template_", '').replace("uid", '')[5:],
                filt_names)

            data = filt_names

        return data
