
//определение классов манипулятор
//Манипулятор позволяет управлять группой объектов в редакторе
//отвечает за перемещение и привязку данных

var Manager = {
    objects: [],
    selector: null,
    setupPanel: null,
    init: function() {
        var me = this;
        this.defineControls();
        //инициализируем опции выбора объектов
        var oltag = '<option>', ortag = '</option>', optstr = '';
        for (var o = 0; o < me.objects_names.length; o++) {
            optstr += [oltag, me.objects_names[o], ortag].join("");
        }
        this.selector.html(optstr);
        this.populateObjects();
    }
};

var LayerManager = function(project, manipulators) {
    var me = this;
    this.autoActivate = true;
    this.project = project;
    this.objects_names = [
        'Поля', 'Блоки'
    ];

    this.manipulators = manipulators;
    this.defineControls = function() {
        //определение контроллов для управления объектами слоя
        var wrapper = $("#select_from_layers");
        this.selector = wrapper.find('select');
        this.activeButton = wrapper.find('button');
        if (this.autoActivate) {
            this.activeButton.css('display', 'none');
            this.selector.on('change', this.activate);
        }
        else {
            this.activeButton.css('display', 'block');
            this.selector.unbind('change', this.activate);
        }
    };
    this.populateObjects = function() {
        this.objects = [];
        var layers = [
            new Layer(),
            new Layer()
        ];
        for (var o = 0; o < layers.length; o++) {
            this.objects.push(layers[o]);
        }
        //ассоциируем слои и манипуляторы
        for (var m = 0; m < this.manipulators.length; m++) {
            var mn = this.manipulators[m];
            mn.layer = this.getLayerById(m);
        }
    };
    this.activate = function(e) {
        var layer_name = me.selector.val(),
            name_index = me.objects_names.indexOf(layer_name),
            object_ = me.objects[name_index];

        me.setActiveLayer(object_);
        //TODO - надо сделать слот для события
    };
    this.setActiveLayer = function(object) {
        //устанавливаем текущий активный слой
        var index_of = this.objects.indexOf(object);
        console.log("SET ACTIVE LAYER -> " + index_of);
        this.deactivateAllLayers();
        object.activate();
        object.visible = true;
    };
    this.deactivateAllLayers = function() {
        //делаем невидимыми все слои
        for (var l=0; l < me.objects.length; l++) {
            var layer = me.objects[l];
            layer.visible = false;
        }
    };
    this.getLayerById = function(id) {
        return me.objects[id];
    };
    this.onLayerMouseDown = function() {
        var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
            manipulator = this.manipulators[active_layer_idx];
        return manipulator.onMouseDown;
    };
    this.onLayerMouseMove = function() {
        var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
            manipulator = this.manipulators[active_layer_idx];
        return manipulator.onMouseMove;
    };
    this.onLayerMouseDrag = function() {
        var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
            manipulator = this.manipulators[active_layer_idx];
        return manipulator.onMouseDrag;
    };
    this.onTemplateSave = function(request_params) {
        /*
            Обработка параметров шаблона перед сохранением на диск
         *  @request_params {Object} - параметры шаблона
         */
        debugger;
        var layer_configurations = [];
        for (var m = 0; m < me.manipulators.length; m++) {
            layer_configurations.push(
                me.manipulators[m].getObjectsConfiguration()
            );
        }

        $.extend(request_params, {
            "layers_configurations": $.toJSON(me.manipulators)
        });
    }

};

var Manipulator = {
    lastPositions: [],
    reversePositions: [],
    viewOnStartup: true,
    init: function() {
        this.defineElements();
        if (!this.viewOnStartup) {
            this.setupElement.css('display', 'none');
        }
        this.showSetup();
        this.createEvents();
    },
    showSetup: function() {
        var me = this,
            checkbox = me.optionShowElement.children(0);
        checkbox.on("change",
            function(e) {
                if (this.control.checked) {
                    me.setupElement.css('display', 'block');
                }
                else {
                    me.setupElement.css('display', 'none');
                }
            }
        )
    },
    getObjectById: function(id) {},
    createEvents: function() {},
    getObjectsConfiguration: function() {
        /*  Шаблонный метод для определения конфигурации объектов слоя

        */
        return {}
    },
    moveBorder: function(border, moveOffset) {
        //перемещение окантовки объекта
        var parent = getParentLine(border),
            object_id = parent[1], object_ = parent[0],
            object = this.getObjectById(object_id);
        if (object) {
            var old_height = object.height,
                old_width = object.width,
                scale_xfactor = 1,
                scale_yfactor = 1,
                x_offset = 0,
                y_offset = 0;

            if (border.position.x == object.lines[0].position.x &&
                border.position.y == object.lines[0].position.y) {
                border.position.y += moveOffset.y;
                object.height += -1* moveOffset.y;
                object.center.y += moveOffset.y / 2;
                scale_yfactor = object.height/old_height;
                y_offset = moveOffset.y / 2;
                object.lines[1].translate(new Point(0, moveOffset.y / 2));
                object.lines[1].scale(scale_yfactor);
                object.lines[3].translate(new Point(0, moveOffset.y / 2));
                object.lines[3].scale(scale_yfactor);
            }
            else if (border.position.x == object.lines[1].position.x &&
                border.position.y == object.lines[1].position.y) {
                border.position.x += moveOffset.x;
                object.width += moveOffset.x;
                object.center.x += moveOffset.x / 2;
                scale_xfactor = object.width/old_width;
                x_offset = moveOffset.x / 2;
                object.lines[0].translate(new Point(moveOffset.x / 2, 0));
                object.lines[0].scale(scale_xfactor);
                object.lines[2].translate(new Point(moveOffset.x / 2, 0));
                object.lines[2].scale(scale_xfactor);
            }
            else if (border.position.x == object.lines[2].position.x &&
                border.position.y == object.lines[2].position.y) {
                border.position.y += moveOffset.y;
                object.height += moveOffset.y;
                object.center.y += moveOffset.y / 2;
                scale_yfactor = object.height/old_height;
                y_offset = moveOffset.y / 2;

                object.lines[1].translate(new Point(0, moveOffset.y / 2));
                object.lines[1].scale(scale_yfactor);
                object.lines[3].translate(new Point(0, moveOffset.y / 2));
                object.lines[3].scale(scale_yfactor);
            }
            else if (border.position.x == object.lines[3].position.x &&
                border.position.y == object.lines[3].position.y) {
                border.position.x += moveOffset.x;
                object.width += (-1) * moveOffset.x;
                object.center.x += moveOffset.x / 2;
                scale_xfactor = object.width/old_width;
                x_offset = moveOffset.x / 2;

                object.lines[0].translate(new Point(moveOffset.x / 2,0));
                object.lines[0].scale(scale_xfactor);
                object.lines[2].translate(new Point(moveOffset.x / 2,0));
                object.lines[2].scale(scale_xfactor);
            }
            //после изменений размеров отдельных ребер надо изменить размер обрамления
            object.boundedRectangle.translate(
                new Point(x_offset,
                    y_offset));
            var scaleY = (object.height - 6) / object.boundedRectangle.bounds.height;
            var scaleX = (object.width - 6) / object.boundedRectangle.bounds.width;
            object.boundedRectangle.scale(scaleX, scaleY);
        }
    },
    moveObject: function(object, moveOffset) {
        //перемещение поля по холсту
        if (this.layer === this.project.activeLayer) {
            var me = this,
                movedObject = me.getParentGroup(object.parent),
                object_id = movedObject[1],
                select_object = movedObject[0],
                object_wrapper = me.getObjectById(object_id),
                object_childs = object_wrapper.getChildrens();

            console.log(select_object);
            for (var c = 0; c < object_childs.length; c++) {
                object_childs[c].translate(moveOffset);
            }
            me.updatePosition(select_object, object_id, me.selected);
            me.updatePositionData(object_id);
        }
    },
    updatePosition: function(obj, obj_id, selected_item) {
        var parent = selected_item.parent;
        obj.updateCenter(obj_id,
            new Point(
                parent.strokeBounds.x + parent.strokeBounds.width / 2,
                parent.strokeBounds.y + parent.strokeBounds.height / 2)
        );
    },
    updatePositionData: function(object_id) {
        updateCoords(object_id);
    },
    setInitalPosition: function(x, y) {
        //устанавливаем начальную позицию для элементов группы
    },
    getCurrentPosition: function() {
        //берем текущую оторбражаемую позицию
    },
    moveFields: function() {

    },
    getFields: function() {
        //возвращает набор которым управляет манипулятор

    },
    addToGroup: function(field) {
        //добавление элемента в группу

    }
};

var FieldManipulator = function(project, layer) {
    //признак того будут ли показана панель настроек при старте
    this.viewOnStartup = false;
    this.project = project;
    this.layer = layer;
    this.segment = null;
    this.rectangle = null;
    this.tooltip_edit = 0;
    this.defineElements = function() {
        this.optionShowElement = $("#show_coordinates_check");
        this.setupElement = $("#coord_info");
    };

};

FieldManipulator.prototype = Manipulator;


var ZoneManipulator = function(project, layer) {
    /*  Зональный манипулятор
        Зона - область в редакторе которая может определять
        местоположение обычных полей (fields). С их помощью
        можно управлять положением групп полей перемещением
        их между различными зонами.
     *  @constructor
     *  @project - объект проект (paper.js)
     *  @layer - объект слоя (paper.js)
     */

    this.project = project;
    this.layer = layer;
    this.viewOnStartup = false;
    this.objects = [];
    this.fn = this.prototype;
    var me = this;
    this.defineElements = function() {
        this.optionShowElement = $('#show_zones_setup_check');
        this.setupElement = $('#zones_setup');
        //элемент представления хранящий информацию о объектах данного слоя
        this.itemsInfoView = this.setupElement.find('table');
        this.setupElement.css('minHeight', '100px');
        this.toolbar = this.setupElement.find('.toolbar');
        this.addBtn = this.toolbar.find('.add_button');
        this.delBtn = this.toolbar.find('.del_button');
        this.addBtn.on('click', this.addZone);
        this.delBtn.on('click', this.delZone);
    };
    this.getParentGroup = function(obj) {
        var om = this.objects;
        for (var n = 0; n < om.length; n++) {
            if (om[n].group_ === obj) {
                return [om[n], n, null];
            }
        }
        return [null, 0, null];
    };
    this.onMouseDrag = function(event) {
        console.log("ZONE MOVED!");
        var object = event.item;
        if (me.isObjectSelected) {
//            console.log("MOVE OBJECT!");
//            debugger;
            me.moveObject(me.selected, event.delta);
        }
        else if (me.isBorderSelected) {
            console.log("MOVE BORDER!");
            me.moveBorder(me.selected, event.delta);
        }
    };
    this.onMouseDown = function(event) {
        console.log("ZONE SELECT!");
        var object = event.item;
        me.getSelectedObject(event.point);
    };
    this.getObjectById = function(object_id) {
        return me.objects[object_id];
    };
    this.deselectAllObjects = function() {
        //снимаем выделение со всех объектов
        me.project.activeLayer.selected = false;
        for (var n =0 ; n < this.objects.length; n++) {
            deselectFunc(this.objects[n]);
        }
    };
    this.isBoundSelected = function(object, select_item_id, all_set) {
        //проверка выбраны ли границы объекта
        var set = null, scnt=0, dcnt=0,
            lines = [
                object.lines[0].id,
                object.lines[1].id,
                object.lines[2].id,
                object.lines[3].id
            ],
            select_in_lines = lines.indexOf(select_item_id);

        if (select_in_lines != -1 && all_set) {
            set = true;
        }
        if (select_in_lines == -1 && !all_set) {
            set = true;
        }

        return set
    };

    this.getSelectedObject = function(hit_point) {
        /* Определение объекта выбора путем анализа результатов коллизий
         *
         */
        var me = this;
        if (this.layer !== this.project.activeLayer) {
            return;
        }
        me.deselectAllObjects();
        var hit_res = me.project.hitTest(hit_point, hitOptions);
        if (!hit_res) {
            return;
        }
        var type = hit_res.type;
        if (hit_res && hit_res.item && !editMode) {

            var selected_item = hit_res.item;

            if (type == 'stroke') {
                selected_item.strokeColor = "#00ff00";
                var rec = me.getParentGroup(selected_item.parent),
                    oid = rec[1], obj = rec[0],
                    object = me.getObjectById(oid);
                console.log(rec);
                //пробуем определить группу к которой принадлежит выбранный элемент
                if (obj != null &&
                    me.isBoundSelected(object, selected_item.id, false)) {
                    //выбран внутренний прямоугольник
                    me.isBorderSelected = false;
                    me.isObjectSelected = true;
                }
                else if (obj != null &&
                    me.isBoundSelected(object, selected_item.id, true)) {
                    //выбран внешний край
                    me.isBorderSelected = true;
                    me.isObjectSelected = false;
                }
            }
            else if (type == 'fill') {
                console.log("SELECT FILL");
                me.isBorderSelected = false;
                me.isObjectSelected = true;
            }
            console.log("SELECT " + type);
            selected_item.selected = true;
            this.selected = selected_item;
        }

    };

    this.addZone = function(btn, width, height, point) {

        var zone_layer = me.layer;
        layer_mgr.setActiveLayer(zone_layer);

        var zwidth = width || 20, zheight = height || 20,
            zpoint = point || new Point(zwidth / 2, zheight / 2),
            zone = createRectangle(zpoint, zwidth, zheight, '#0066ff',
                {showTooltip: false, manager: me.objects}
            );

        me.objects.push(zone);
        me.createObjectInfo();
    };

    this.delZone = function() {
        var zone = null;
        me.objects.remove(zone);
    };

    this.getModelFields = function() {
        return ['name', 'position', 'childs'];
    };

    this.getObjectInfo = function(id) {
        var fields = me.getModelFields(),
            row = this.itemsInfoView.find('tr').eq(id),
            columns = row.find('td'), info = {}, field_counter = 0;

        columns.each(function(c) {
            info[fields[++field_counter]] = c.html();
        });
        return info;
    };

    this.createObjectInfo = function() {
        debugger;
        var fields = me.getModelFields(),
            row = this.itemsInfoView.append('<tr></tr>').find('tr').eq(-1);

        for (var f = 0; f < fields.length; f++) {
            var column = row.append('<td></td>').find('td').eq(-1);
            column.html('<input name="' + fields[f] + '" />');
        }
    };

    this.getObjectsConfiguration = function() {
        var rows = this.itemsInfoView.find('tr'),
            config = [];
        rows.each(function(r) {
            var columns = r.children(),
                fields = me.getModelFields(),
                row_info = {}, field_counter = 0;
            columns.each(function(c) {
                row_info[fields[++field_counter]] = c.val();
                config.push(row_info);
            });
        });
        return config;
    };

    this.fixToZone = function() {
        //установка группы элементов в определенную зону

    };
    this.updatePositionData = function(object_id) {
        //меняем информацию по расположению зон
    };
    return this;
};

ZoneManipulator.prototype = Manipulator;

LayerManager.prototype = Manager;
