/*конфигурация зависимостей для RequireJS*/

requirejs.config({
    baseUrl: staticUrl + 'js/libs',
    paths: {
        jquery          : 'jquery',
        jquery_ui       : 'jquery-ui-1.10.3',
        json            : 'json-2.4',
        handlebars      : 'handlebars-v1.3.0',
        spec            : '../test/jasmine/spec/',
        tech            : '../tech',
        cells           : '../tech/cells',
        drawing         : '../tech/drawing',
        hyphenation     : '../tech/hyphenation',
        helpers         : '../tech/helpers',
        toolpanel       : '../tech/toolpanel',
        models          : '../tech/models', 
        views           : '../tech/views',
        collections     : '../tech/collections',
        // akordeon        : 'jquery.akordeon'
//        zones:          '../tech/zones'
    },
    shim: {
        'backbone': {
            deps: ['underscore', 'jquery'],
            exports: 'Backbone'
        },
        'underscore': {
            exports: '_'
        },
        'paper': {
            exports: 'Paper'
        },
        'handlebars-v1.3.0': {
            exports: 'handlebars'
        },
        // 'jquery-ui-1.10.3': {
        //     exports: 'jquery_ui',
        //     deps: ['jquery']
        // },
        'json-2.4': {
            exports: 'json',
            deps: ['jquery']
        },
        jasmine: {
            exports: 'jasmine'
        },
        'jasmine-html': {
            deps: ['jasmine'],
            exports: 'jasmine'
        }
        // 'akordeon': {
        //     exports: 'JQuery.fn.akordeon',
        //     deps: ['jquery']
        // }
    }
});

BlankEditor = (function() {
    var blankEditor = {};

    requirejs(['paper',
        'views/editor_view', 'models/editor_model',
        'views/toolbox_view',
        'views/cellinfo_view',
        'collections/cellmanager',
        'drawing/drawer',
        'views/tooltip_view',
        'views/template_manage_view',
        'views/offset_setup_view',
        'views/font_setup_view',
        'views/layer_manager_view'
    ],

        function(Paper, EditorView, EditorModel, ToolboxView,
                 CellInfoView, CellCollection, Drawer,
                 TooltipView,
                 TemplateManageView,
                 OffsetSetupView,
                 FontSetupView,
                 LayerManagerView
            ) {

            var body = $('body'),
                options = {
                    offsetControl: false
                };

            var editor_model = new EditorModel().set(
                window.editor_config
            );

            var defaultPubSub = _.extend({}, Backbone.Events);

            var editor_view = new EditorView({
                pubSub: defaultPubSub,
                model: editor_model
            });

            var changeIf = function (param_name, val) {
                var old_val = this.model.get(param_name);
                if (parseFloat(old_val) == parseFloat(val)) return;

                if (!this.model.get('offsetMode')) {
                    this.model.set(param_name, parseFloat(val));
                }
            };
            //слушаем события при изменении отступов и масштабов (используем шину pubSub)
            editor_view.listenTo(editor_view.pubSub, 'scale_x:set', function(val) {
                changeIf.call(this, 'fix_x', val);
            }, editor_view);

            editor_view.listenTo(editor_view.pubSub, 'scale_y:set', function(val) {
                changeIf.call(this, 'fix_y', val);
            }, editor_view);

            editor_view.listenTo(editor_view.pubSub, 'top_offset:set', function(val) {
                changeIf.call(this, 'offset_y', val);
            }, editor_view);

            editor_view.listenTo(editor_view.pubSub, 'left_offset:set', function(val) {
                changeIf.call(this, 'offset_x', val);
            }, editor_view);

            body.append(editor_view.render().el);
            editor_view.afterRender();
            //рендерим представление для панели инструментов
            var toolbox_view = new ToolboxView({
                pubSub: defaultPubSub,
                editor: editor_view
            });
            body.append(toolbox_view.render().el);
            toolbox_view.afterRender();

            var cellCollection = new CellCollection();

            _.extend(cellCollection, {
               number_of_params    : editor_view.model.get('number_of_params'),
               coord_map           : editor_view.model.get('coord_map'), 
               user_map            : editor_view.model.get('user_map'),
               user_fields         : editor_view.model.get('user_fields')
            });

            var report_params = editor_model.get('report_params');

            //менеджер слоев отвечает за заполнение объектов данными
            //и управляет видимостью слоев

            var layerManager = new LayerManagerView(
                {
                    pubSub: defaultPubSub,
                    drawer: Drawer,
                    editor: editor_view,
                    objectsCollection: cellCollection,
                    loadDataUrl: editor_view.getActions().get_params_url,
                    loadDataParams: {
                        pack_url: editor_view.getActions().provider_url,
                        id_field: report_params.id_field,
                        add_vals: report_params.additional_values
                    }
                });

            toolbox_view.$el.append(layerManager.render().el);

            //инициализируем представление для работы с шаблонами
            var template_view = new TemplateManageView({
                pubSub: defaultPubSub,
                editor: editor_view,
                objectsCollection: cellCollection,
                template_name: editor_model.get('template_name')
            });
            toolbox_view.$el.append(template_view.render().el);
            template_view.afterRender();
            //загружаем список шаблонов которые были созданы пользователем
            var promise = template_view.preloadAllTemplates();
//        promise.done(template_view.setTemplateToLoad, template_view);

            if (options.offsetControl) {
                var offset_setup_view = new OffsetSetupView({
                    editor_model: editor_model,
                    pubSub: defaultPubSub
                });

                toolbox_view.$el.append(offset_setup_view.render().el);
            }

            var font_setup_view = new FontSetupView({
                model: editor_model,
                pubSub: defaultPubSub
            });

            toolbox_view.$el.append(font_setup_view.render().el);
            font_setup_view.afterRender();

            var celltable_view = new CellInfoView({
                pubSub: defaultPubSub,
                collection: cellCollection
            });

            toolbox_view.$el.append(celltable_view.render().el);
            celltable_view.afterRender();

            celltable_view.listenTo(celltable_view.pubSub, 'font_params:apply',
                celltable_view.setFontParams);
        	//меняем текст описания при изменении его в модели
        	cellCollection.on('change:desc', celltable_view.updateCellDescription, celltable_view);
            // toolbox_view.$el.accordion({collapsible: true});

            paper.install(window);
            paper.setup(editor_view.canvas.attr("id"));
            //перехватываем события мыши при перемещении, нажатии и перетаскивании
            Drawer.processCellEvents(cellCollection);
            //обратные события от ручного задания положения ячеек
            Drawer.processOuterCellEvents(Drawer, celltable_view);
            //ловим события при изменении в коллекции
            celltable_view.collection.on('add', Drawer.drawCell);
            celltable_view.collection.on('remove', Drawer.eraseCell);

            //при переключении печати второй стороны к ячейкам нужно добавить
            //оффсет
            editor_model.on('change:twoSideOffset', function(model, val, options) {
                var coll = this.collection;

                coll.each(function(cell) {
                    //уменьшаем смещение на половину от краев холста
                    cell.set('offsetType', 'absolute');
                    cell.set('printOffsetTop', val);
                });

            }, celltable_view);

            //когда новая ячейка добавляется на холст нужно правильно заполнить
            //цветом в зависимости от выбранного режима редактора
            celltable_view.collection.on('draw_complete', Drawer.deselectFunc, editor_view);

            //представление отвечает за изменение режима отступов если из внешнего представления
            //приходит данное событие
            editor_view.listenTo(editor_view.pubSub, 'offset_mode:change',
                editor_view.setOffsetParams);
            //ловим события при изменениях масштаба и оступа
            editor_model.on('change:offsetMode', function() {

                var top_offset, left_offset, nscale_x, nscale_y,
                    offsetMode = this.get('offsetMode');

                top_offset = this.get('offset_y');
                left_offset = this.get('offset_x');
                nscale_x = this.get('fix_x');
                nscale_y = this.get('fix_y');

                //если режим выключен инвертируем значения отступов и масштабов
                if (!offsetMode) {
                    nscale_x = 1 / nscale_x;
                    nscale_y = 1 / nscale_y;
                    top_offset = (-1) * top_offset;
                    left_offset = (-1) * left_offset;
                }

                var range = [0, cellCollection.length];
                var selected_ids = celltable_view.getSelectedIds();

                if (top_offset) {
                    cellCollection.bulkSetParams(Drawer, selected_ids, "ot", top_offset);
                }
                if (left_offset) {
                    cellCollection.bulkSetParams(Drawer, selected_ids, "ol", left_offset);
                }

                if (nscale_x != 1 || nscale_y != 1) {
                    var chained = {};

                    cellCollection.bulkSetParams(Drawer, selected_ids, "sw", nscale_x, chained);
                    // cellCollection.bulkSetParams(Drawer, selected_ids, "sl", nscale_x, chained);
                    cellCollection.bulkSetParams(Drawer, selected_ids, "sh", nscale_y, chained);
                    // cellCollection.bulkSetParams(Drawer, selected_ids, "st", nscale_y, chained);

                    if (chained) {
                        cellCollection.bulkChained(Drawer, chained);
                    }
                }
                //обновляем лейблы
                cellCollection.bulkSetParams(Drawer, selected_ids, 'label');
            });

            var showOrHideTooltip = function(selectionModel, forceHide) {

                if (Drawer.tooltipIsOn) {
                    TooltipView.hideTooltip(selectionModel);
                    Drawer.tooltipIsOn = false;
                }

                if (forceHide) {
                    return;
                }

                if (!Drawer.tooltipIsOn) {
                    setTimeout(function() {
                        if (!selectionModel.get('dragMode')) {
                            var tooltip = new TooltipView({
                                selectionModel: selectionModel,
                                cellCollection: cellCollection
                            }),
                                rendered_view = tooltip.render().el;

                            body.append(rendered_view);
                            tooltip.afterRender();
                            //добавим опции выбора шрифтов и размеров
                            tooltip.addFontTypeOptions(font_setup_view.fontSelector);
                            tooltip.addFontSizeOptions(font_setup_view.fontSizeSelect);
                            //загружаем данные в объект подсказки
                            tooltip.tooltipLoadInfo();
                            Drawer.tooltipIsOn = tooltip;

                            var onCloseClosure = function() {
                                onCloseClosure.Drawer.tooltipIsOn = false;
                            };
                            onCloseClosure.Drawer = Drawer;

                            tooltip.$el.dialog({
                                'close': onCloseClosure
                            });
                        }
                    }, 200);

                } else {
                    TooltipView.hideTooltip(selectionModel);
                    Drawer.tooltipIsOn = false;
                }
            };
            var renderTooltip = function(sm) {showOrHideTooltip(sm);},
                hideTooltip = function(sm) {showOrHideTooltip(sm, true);};

            Drawer.selectionModel.on('select_line', renderTooltip);
            Drawer.selectionModel.on('select_border', renderTooltip);
            Drawer.selectionModel.on('select_fill', renderTooltip);
            Drawer.selectionModel.on('unselect', hideTooltip);

            //подписываемся на первое событие добавления всех ячеек и
            //далее нужно отписаться от этого события
            cellCollection.on('cells_done', function() {
                var collection = this;
                editor_model.set('editMode', false);
                collection.off('cells_done');
            });
            //добавляем ранее сохраненные ячейки
            cellCollection.addCellsFromTemplate();
            // toolbox_view.$el.akordeon();
            BlankEditor.EditorModel = editor_model;
            BlankEditor.EditorView = editor_view;
            BlankEditor.Drawer = Drawer;
            BlankEditor.CellCollection = cellCollection;
            BlankEditor.CellTable = celltable_view;
            BlankEditor.FontSetupView = font_setup_view;
            BlankEditor.TemplateSetupView = template_view;
            BlankEditor.LayerSetupView = layerManager;
        }
    );

    return blankEditor;
})();


//отладочная функция для вывода данных по полям
function debugSoft() {
    console.clear();
    console.log("***---LAYER DATA->");
    for (var r in project.activeLayer.children){
        if (project.activeLayer.children[r] instanceof Group) {
            console.log(project.activeLayer.children[r]);
        }
    }
    console.log("***---ELEMENTS ON LAYER->");
    console.log(r);
    console.log("***---FIELDS_DATA->");
    for (var r in recs) {

        console.log(recs[r]);
    }
    console.log("***---PRIMITIVES IN ARRAY->");
    console.log(r);
}