
define('collections/cellmanager', ['jquery', 'helpers', 'backbone', 'paper', 'models/cell_model'],

function($, helpers, Backbone, Paper, CellModel) {

    var CellCollection = Backbone.Collection.extend({

        modelType: CellModel,

        initialize: function(config) {

            if (config) {
                this.cell_values = config ['values'];
                this.cell_descriptions = config['descriptions'];
            }

            if (!this.cell_values) {
                this.cell_values = this.extractValues();
            }
            if (!this.cell_descriptions) {
                this.cell_descriptions = this.extractDescriptions();
            }
        },
        getParentLine: function (segment) {
            var ret;

            this.each(function(item) {
                var group = item.get('group_');
                if (group && group.isChild(segment)) {
                    ret = [item, item.get('idx'), null];
                }
            });

            if (ret) {
                return ret;
            } else {
                return [null, 0, null];
            }
        },
        getParentGroup: function (obj) {

            var ret;

            this.each(function(item) {
                var group = item.get('group_');
                if (group && group.id == obj.id) {
                    ret = [item, item.get('idx'), null];
                }
            });

            if (ret) {
                return ret;
            } else {
                return [null, 0, null];
            }
        },
        extractValues: function() {
            //извлекаем из шаблона текст для ячеек
            var cell_values = [],
                values = $(".hidden_data");

            values.each(function(e) {
                if (e < number_of_params) {
                    cell_values.push($(this).html());
                }
            });

            return cell_values;
        },
        extractDescriptions: function() {
            //извлекаем из шаблона описание для ячеек
            var cell_descriptions = [],
                descriptions = $(".hidden_data1");

            descriptions.each(function(e) {

                if (e < number_of_params) {
                    cell_descriptions.push($(this).html());
                }

            });

            return cell_descriptions;
        },
        addToPermCollection: function (cells_count) {
            var coord_map = this.coord_map, new_cell_config;

            for (var i = 0; i < cells_count; i++) {
                new_cell_config = {
                    idx: i + 1,
                    desc: this.cell_descriptions[i],
                    base_text: this.cell_values[i],
                    fixed: true
                };
                console.log("CREATE CELL");
                console.dir(new_cell_config);
                this.view_.addCell(null, new_cell_config);

            }
        },
        addToUserCollection: function () {

            var coord_map = this.coord_map,
                user_map = this.user_map,
                user_fields = this.user_fields, new_cell_config;

            for (var j = coord_map.length; j < (coord_map.length + user_map.length); j++) {

                user_fields.push(j);
                var in_map_id = j - coord_map.length,
                new_cell_config = {
                    idx: j, user_idx: in_map_id,
                    desc: this.cell_descriptions[j],
                    base_text: this.cell_values[j],
                    fixed: false
                };
                this.view_.addCell(null, new_cell_config);
            }
        },
        addCellsFromTemplate: function() {
            
    //        var layer = layer_mgr.getLayerById(0);
    //        layer_mgr.setActiveLayer(layer);

            //Глобальные настройки редактора
            var number_of_params = this.number_of_params,
                coord_map = this.coord_map,
                user_map = this.user_map,
                user_fields = this.user_fields;

            //вычислим количество искусственно добавленных ячеек
            var additional_fields = number_of_params - coord_map.length,
                default_map_size;

            if (additional_fields < 0) {
                additional_fields = (-1) * additional_fields;
            }

            default_map_size = coord_map.length + additional_fields;

            //добавляем перманентые поля (те которые добавлены автоматически)
            this.addToPermCollection(default_map_size);
            //добавляем поля которые добавлены пользователем искусственно
            this.addToUserCollection();

            this.trigger('cells_done', this);
        },
        bulkSetParams: function(drawer, range, param_name, value, chained) {
            //задание значение одного из параметров для нескольких ячеек одновременно
            // @param range - массив из порядковых номеров ячеек
            // @chained - признак того что нужно только определить параметры новых 
            // объектов и по окончанию цепочки применить
            //ПРИМЕР ИСПОЛЬЗОВАНИЯ
            //- bulkUpdateParams([1,30],"w","10.5")
            //изменение ширины для полей с номера с 1 по 30 на значение 10.5

            var cell, center, width, height, val, only_calculate;
            //предполагается что значение задано в метрических единицах (мм)
            for (var i = 0; i < range.length; i++) {

                cell = this.at(range[i]);
                width = cell.get('width');
                height = cell.get('height');
                center = cell.get('center');
                only_calculate = chained ? true : false;

                if (param_name == "w") { //ширина
                    drawer.updateRWidth.call(cell, value, only_calculate);
                }
                else if (param_name == "h") { //высота
                    drawer.updateRHeight.call(cell, value, only_calculate);
                }
                else if (param_name == "ol") { //смещение по левой границе
                    var converted_value = helpers.to_pUnit(value);
                    drawer.updateCenterX.call(cell, converted_value, only_calculate);
                }
                else if (param_name == "ot") { //смещение по правой границе
                    var converted_value = helpers.to_pUnit(value);
                    drawer.updateCenterY.call(cell, converted_value, only_calculate);
                }
                else if (param_name == "sw") {
                    var swidth = width * value;
                    val = drawer.updateRWidth.call(cell, swidth, only_calculate);
                }
                else if (param_name == "sh") {
                    var sheight = height * value;
                    val = drawer.updateRHeight.call(cell, sheight, only_calculate);
                }
                else if (param_name == "sl") {

                    var left_offset = center.x - width/2,
                        val1 = left_offset * value,
                        delta = val1 - left_offset;

                    console.log("LEFT OFFSET->" + helpers.to_hUnit(val1) + "mm, " + "=" + helpers.to_hUnit(delta));
                    val = drawer.updateCenterX.call(cell, delta, only_calculate);
                }
                else if (param_name=="st") {
                    var top_offset = center.y - height/2,
                        val1 = top_offset * value,
                        delta = val1 - top_offset;

                    val = drawer.updateCenterY.call(cell, delta, only_calculate);
                }
                else if (param_name == "label") {
                    drawer.updateLabel.call(cell);
                }

                if (val && chained) {
                    if (chained[i]) {
                        $.extend(true, chained[i], val);
                    } else {
                        chained[i] = val;
                    }
                }
            }
        },
        bulkChained: function(drawer, chained) {
            var has = Object.prototype.hasOwnProperty,
                cell, center, set;

            for (var c in chained) {
                if (has.call(chained, c)) {
                    cell = this.at(c);

                    if (chained[c].set) {
                        set = chained[c].set;
                        delete chained[c].set;

                        var center = cell.get('center');
                        if (set.center_x) {
                            center.x = set.center_x.x;
                            delete set.center_x;
                        }
                        if (set.center_y) {
                            center.y = set.center_y.y;
                            delete set.center_y;
                        }

                        for (var s in set) {
                            if (has.call(set, s)) {
                                cell.set(s, set[s]);
                            }
                        }

                        cell.set('center', center);
                    }

                    for (var p in chained[c]) {
                        if (has.call(chained[c], p)) {
                            cell.set(p, chained[c][p]);
                        }
                    }

                }

                // if (center) {
                //     chained[c]['delta_x'] = center.x;
                //     chained[c]['center_y'] = center.y;
                // }
                drawer.updateRectangleGeometry(cell, chained[c]['cell_copy'], chained[c]);
            }
        }

    });

    return CellCollection
});