/**
 *  Модуль рисования ячеек на холсте
 *  К основным функциям относятся рисование прямоугольника, метки, выбор ячейки, снятие выделения
 *  createRectangle,
 *  createSegmentRectangle,
 *  createFieldLabel,
 *  deselectFunc
 */

define('drawing/drawer', ['jquery', 'paper', 'models/selection_model', 'helpers'],

function($, Paper, SelectionModel, helpers) {

    var fieldLabelDefaultFont = 'Arial',
        fieldLabelDefaultSize = 12,
        selection_model = new SelectionModel();

    function update_BRecGeometry(cell, xofs, yofs) {

        var bound = cell.get('boundedRectangle'), 
            group, center, width, height, scaleX, scaleY, label;

        width = cell.get('width'); height = cell.get('height');
        center = cell.get('center'); group = cell.get('group_');
        scaleX = (width - 6)/ bound.bounds.width;
        scaleY = (height - 6)/ bound.bounds.height;

        bound.scale(scaleX, scaleY);
        bound.translate(new Point(xofs, yofs));

    }

    function updateRectangleGeometry(cell, old_cell, opt_geometry) {
        //если передан параметр opt_geometry то модифицируем прямоугольник в соответствии с новыми параметрами opt_geometry
        //также надо переместить в новую позицию объемлющий прямоугольник
        var cell_width = cell.get('width'),
            cell_height = cell.get('height'),
            cell_center = cell.get('center'),
            cell_group = cell.get('group_'),
            lines = cell.get('lines'),
            delta_x = -0, delta_y = -0,
            scale_x = 1, scale_y = 1,
            delta_point, dlp, dln;

        if (opt_geometry.delta_x) {
            delta_x = opt_geometry.delta_x;
        }
        if (opt_geometry.delta_y) {
            delta_y = opt_geometry.delta_y;
        }

        //ширина меняется относительно левого края
        if (opt_geometry.width) {
            scale_x = cell_width / old_cell.get('width');
        }
        //высота меняется относительно верхнего края
        if (opt_geometry.height) {
            scale_y = cell_height / old_cell.get('height');
        }

        delta_point = new Point(delta_x, delta_y);

        if (scale_y != 1) {

            //изменение высоты прямоугольника
            lines[1].scale(1, scale_y);
            lines[3].scale(1, scale_y);

            dlp = new Point(0, (
                lines[1].position.y
                + lines[1].bounds.height/2)
                - lines[2].position.y);

            dln = new Point(0, (
                lines[1].position.y
                - lines[1].bounds.height/2)
                - lines[0].position.y);
            //смещаем прямоугольник от верхнего края
            delta_point = new Point(0, dlp.y);

            lines[0].translate(dln);
            lines[2].translate(dlp);

            update_BRecGeometry(cell, 0, 0);
        }

        if (scale_x != 1) {
            //изменение ширины прямоугольника
            lines[0].scale(scale_x, 1);
            lines[2].scale(scale_x, 1);

            dlp = new Point((lines[0].position.x - lines[0].bounds.width / 2)
                - lines[3].position.x, 0);

            dln = new Point((lines[0].position.x + lines[0].bounds.width / 2)
                -  lines[1].position.x, 0);
            //смещаем прямоугольник от левого края
            delta_point = new Point(dln.x, 0);

            lines[1].translate(dln);
            lines[3].translate(dlp);
            update_BRecGeometry(cell, 0, 0);
        }

        for (var chld in cell_group.children) {
            cell_group.children[chld].translate(delta_point);
        }

        var bound = cell.get('boundedRectangle');

        bound.width = cell_width;
        bound.height = cell_height;

    }

    function drawCell(model, options) {

        var link = [-1, 0],
            cell_text_params = model.get('text_params'),
            id = parseInt(model.get('idx')),
            map_id = id - 1,
            new_cell_config;

        new_cell_config = createFieldPrototype(map_id, options.coord_map);

        link[0] = getLayerParams(id, "linked", options.coord_map);

        //ищем залинкованные поля в конце массива
        if (link[0] != -1) {
            updateCellLinks(model, link);
        } else {
            updateTextParams(model, options.cell_values[map_id]);
        }

        new_cell_config.linked_id = link;
        new_cell_config.descr = options.cell_descriptions[map_id];

        model.set(new_cell_config);

        cell_text_params.font_type = new_cell_config.font_type;
        cell_text_params.font_size = new_cell_config.font_size;

        model.trigger('draw_complete', model);

    }

    function eraseCell() {
        //полное удаление объекта с холста
        //TODO - допилить метод
    }

    function getLayerParams(field_idx, param_name, map) {

        var param_order = {
            'center_x'  : {"idx": 0, "default_value": 70 },
            'center_y'  : {"idx": 1, "default_value": (70 + field_idx * 20 + 1)},
            'width'     : {"idx": 2, "default_value": 20 },
            'height'    : {"idx": 3, "default_value": 20 },
            'print_off' : {"idx": 4, "default_value": 0  },
            'linked'    : {"idx": 5, "default_value": -1 },
            'font_type' : {"idx": 6, "default_value": "Arial" },
            'font_size' : {"idx": 7, "default_value": "12" }
        };

        var field_params = map[field_idx],
            map_idx = param_order[param_name] ?
            param_order[param_name]["idx"] : null,
            val = null;

        if (map_idx >= 0 && field_idx < map.length
            && field_params.length > map_idx) {
            val = field_params[map_idx];
        } else if (map_idx >= 0) {
            val = param_order[param_name]["default_value"];
        }

        return val;
    }

    var createFieldPrototype = function (map_id, map) {

        var i = map_id,
            new_center_x = 0, new_center_y = 0,
            new_width = 0, new_height = 0,
            print_off = 0,
            //TODO - убрать во вьюхи
            font_type = $("#font_selector").val(),
            font_size = $("#selector_fontsize").val(),
            new_field;

        if (map.length > 0) {
            new_center_x    = getLayerParams(i, "center_x", map);    //центр Х поля
            new_center_y    = getLayerParams(i, "center_y", map);    //центр Y поля
            new_width       = getLayerParams(i, "width", map);       //ширина поля
            new_height      = getLayerParams(i, "height", map);      //высота поля
            print_off       = getLayerParams(i, "print_off", map);

            //находим в таблице индивидуальные настройки
            //по шрифтам
            //для совместимости со старыми шаблонами
            if (getLayerParams(i, "font_type", map) &&
                getLayerParams(i, "font_size", map)) {
                font_type = getLayerParams(i, "font_type", map);
                font_size = getLayerParams(i, "font_size", map);
            }
            //правки к тому что положение задается от левого края и верхнего края холста
            new_center_x += new_width/2;
            new_center_y += new_height/2;
        }

        if (!new_width) new_width = 20;
        if (!new_height) new_height = 20;
        if (!new_center_x) new_center_x = 70;
        if (!new_center_y) new_center_y = 70 + i * new_height + 1;

        var center = new Point(new_center_x, new_center_y);

        new_field = createSegmentRectangle(center, new_width, new_height, i);

        if (print_off == 0) new_field.print = true;
        else if (print_off == 1) new_field.print = false;

        new_field.font_type = font_type;
        new_field.font_size = font_size;

        return new_field
    };

    var updateTextParams = function (cell, text) {
        //изменяем текстовые параметры ячейки
        var cell_text_params = cell.get('text_params');
        //модифицируем параметры ячейки
        if (text) {
            cell_text_params.rows.push(text);
            cell_text_params.rows_offset.push(0);
            if (text) {
                cell.set('base_text', text);
            }
        
        } else {
            cell_text_params.rows = [];
            cell_text_params.rows_offset = [];
        }
        cell.set('text_params', cell_text_params);
    };

    var onMouseDownHandler = function(event) {

        //    var layer_specific_handler = layer_mgr.onLayerMouseDown();
        //
        //    if (layer_specific_handler) {
        //        layer_specific_handler(event);
        //        return;
        //    }
        var that = selection_model,
            collection = onMouseDownHandler.collection,
            hitOptions = that.get('hitOptions'),
            editMode = that.get('editMode');

        project.activeLayer.selected = false;

        that.set('selected_item', null);
        that.set('segment', null);
        that.set('rectangle', null);

        var hit_res = project.hitTest(event.point, hitOptions);

        //снимаем выделение со всех ячеек
//        collection.each(function(cell) {
//            console.log();
//            deselectFunc(cell, editMode);
//        });

        //модуль подсказки должен скрывать tooltip
        selection_model.trigger('before_select', selection_model);

        if (hit_res && hit_res.item && !editMode) {

            that.set('selected_item', hit_res.item);

            if (hit_res.type == 'stroke') {

                hit_res.item.strokeColor = "#00ff00";

                var rec = collection.getParentGroup(hit_res.item.parent),
                    select_rectangle = rec[0] ? collection.find(
                    function(cell) {
                        return cell.get('idx') == parseInt(rec[1]);
                    }) : null;

                //пробуем определить группу к которой принадлежит выбранный элемент
                if (select_rectangle) {
                    var lines = select_rectangle.get('lines'),
                        border_lines_id = [
                            lines[0].id,
                            lines[1].id,
                            lines[2].id,
                            lines[3].id
                        ];
                    //выбран внутренний прямоугольник
                    if (border_lines_id.indexOf(hit_res.item.id) === -1) {
                        that.set('rectangle', hit_res.item);
                        selection_model.trigger('select_border', that);
                    }//выбран внешний край
                    else if (border_lines_id.indexOf(hit_res.item.id) != -1) {
                        that.set('segment', hit_res.item);
                        selection_model.trigger('select_line', that);
                    }

                    // if (rec[0].showTooltip) {
                    //     showTooltip(rec[0].get('boundedRectangle'));
                        // tooltip_edit.tooltip_edit = 1;
                    // }
                }
            }
            else if (hit_res.type == 'fill') {

                that.set('rectangle', hit_res.item);
                selection_model.trigger('select_fill', that);

                // showTooltip(this.rectangle);
                // tooltip_edit.tooltip_edit = 1;
            }

            hit_res.item.selected = true;
        } else {
            selection_model.trigger('unselect', that);
        }
    };

    var onMouseMoveHandler = function(event) {


        var sel_model = selection_model,
            collection = onMouseMoveHandler.collection,
            hitOptions = sel_model.get('hitOptions'),
            hit_res = project.hitTest(event.point, hitOptions),
            tooltip_edit = sel_model.get('tooltip_edit'),
            editMode = sel_model.get('editMode');

            if (sel_model.get('dragMode')) {
                sel_model.set('dragMode', false);
            }
//            var layer_specific_handler = layer_mgr.onLayerMouseMove();
//
//            if (layer_specific_handler) {
//                layer_specific_handler(event);
//                return;
//            }

        project.activeLayer.selected = false;

        if (hit_res && hit_res.item && !editMode) {

            // if (!tooltip_edit) {
            //     showTooltip(hit_res.item);
            // }

            if (hit_res.type == 'stroke') {
                hit_res.item.selected = true;
                sel_model.trigger('move_line', sel_model);
            }
        }
        else {
            project.activeLayer.selected = false;
            //скрытие подсказки

            // var tooltip = that.tooltip;
            // if (tooltip && !tooltip_edit) {
            //     tooltip.remove();
            //      tooltip = null;
            // }

//            sel_model.trigger('unselect', sel_model);

//            collection.each(function(cell) {
//                deselectFunc(cell, editMode);
//            })
        }
    };

    var onMouseDragHandler = function(event) {

        var sel_model = selection_model,
            collection = onMouseDragHandler.collection,
            editMode = sel_model.get('editMode'),
            rectangle = sel_model.get('rectangle'),
            segment = sel_model.get('segment');

            if (!sel_model.get('dragMode')) {
                sel_model.set('dragMode', true);
            }
        //    var layer_specific_handler = layer_mgr.onLayerMouseDrag();
//
//            if (layer_specific_handler) {
//                layer_specific_handler(event);
//                return;
//            }

        if (segment && !editMode) {


            var parent = collection.getParentLine(segment),
                segment = sel_model.get('segment');

            if (parent[0]) {
                var frag = collection.find(
                    function(cell) {
                        return cell.get('idx') == parseInt(parent[1]);
                    }),
                    frag_lines = frag.get('lines'),
                    old_height = frag.get('height'), new_height,
                    old_width = frag.get('width'), new_width,
                    old_center = frag.get('center'), new_center_y, new_center_x,
                    scale_xfactor = 1, scale_yfactor = 1,
                    x_offset = 0, y_offset = 0;

                if (segment.position.x == frag_lines[0].position.x &&
                    segment.position.y == frag_lines[0].position.y) {

                    segment.position.y += event.delta.y;

                    new_height = old_height + (-1* event.delta.y);
                    new_center_y = frag.get('center').y + event.delta.y/2;

                    old_center.y = new_center_y;
                    frag.set('height', new_height);
                    frag.set('center', old_center);

                    scale_yfactor = new_height / old_height;
                    y_offset = event.delta.y/2;

                    frag_lines[1].translate(new Point(0, event.delta.y/2));
                    frag_lines[1].scale(scale_yfactor);
                    frag_lines[3].translate(new Point(0, event.delta.y/2));
                    frag_lines[3].scale(scale_yfactor);

                } else if (segment.position.x == frag_lines[1].position.x &&
                           segment.position.y == frag_lines[1].position.y) {

                    segment.position.x += event.delta.x;

                    new_width = old_width + event.delta.x;
                    new_center_x = old_center.x + event.delta.x / 2;

                    old_center.x = new_center_x;

                    frag.set('width', new_width);
                    frag.set('center', old_center);

                    scale_xfactor = new_width / old_width;
                    x_offset = event.delta.x / 2;

                    frag_lines[0].translate(new Point(event.delta.x/2,0));
                    frag_lines[0].scale(scale_xfactor);
                    frag_lines[2].translate(new Point(event.delta.x/2,0));
                    frag_lines[2].scale(scale_xfactor);

                } else if (segment.position.x == frag_lines[2].position.x &&
                           segment.position.y == frag_lines[2].position.y) {

                    segment.position.y += event.delta.y;

                    new_height = old_height + event.delta.y;
                    new_center_y = old_center.y + event.delta.y / 2;

                    old_center.y = new_center_y;

                    frag.set('height', new_height);
                    frag.set('center', old_center);

                    scale_yfactor = new_height / old_height;
                    y_offset = event.delta.y / 2;

                    frag_lines[1].translate(new Point(0, event.delta.y / 2));
                    frag_lines[1].scale(scale_yfactor);
                    frag_lines[3].translate(new Point(0, event.delta.y / 2));
                    frag_lines[3].scale(scale_yfactor);

                    //сдвигаем метку также
                    var cellMark = frag.get('id_text');
                    cellMark.translate(new Point(0, event.delta.y));
                }
                else if (segment.position.x == frag_lines[3].position.x &&
                         segment.position.y == frag_lines[3].position.y) {

                    segment.position.x += event.delta.x;

                    new_width = old_width + ((-1) * event.delta.x);
                    new_center_x = old_center.x + event.delta.x / 2;

                    old_center.x = new_center_x;

                    frag.set('width', new_width);
                    frag.set('center', old_center);

                    scale_xfactor = new_width / old_width;
                    x_offset = event.delta.x / 2;

                    frag_lines[0].translate(new Point(event.delta.x / 2, 0));
                    frag_lines[0].scale(scale_xfactor);
                    frag_lines[2].translate(new Point(event.delta.x / 2, 0));
                    frag_lines[2].scale(scale_xfactor);

                    //сдвигаем метку также
                    var cellMark = frag.get('id_text');
                    cellMark.translate(new Point(event.delta.x, 0));
                }

                //после изменений размеров отдельных ребер надо изменить размер обрамления
                var bound = frag.get('boundedRectangle'),
                    post_width = frag.get('width'),
                    post_height = frag.get('height'),
                    scaleX = (post_width - 6) / bound.bounds.width,
                    scaleY = (post_height - 6) / bound.bounds.height;

                bound.translate(new Point(x_offset, y_offset));               
                bound.scale(scaleX, scaleY);

                frag.set('boundedRectangle', bound);
            }
        }

        if (rectangle && !editMode) {

            //ПЕРЕМЕЩЕНИЕ ПОЛЯ ПО СЛОЮ
            var movedObject = collection.getParentGroup(rectangle.parent),
                frag = collection.find(
                    function(cell) {
                        return cell.get('idx') == parseInt(movedObject[1]);
                    }),
                frag_group = frag.get('group_');

            if (frag_group) {
                //ПЕРЕНОС ОСУЩЕСТВЛЯЕТСЯ НЕ ПЕРЕНОСОМ ГРУППЫ, А ПЕРЕНОСОМ ЭЛЕМЕНТОВ В ГРУППЕ
                for (var chld in frag_group.children) {
                    frag_group.children[chld].translate(event.delta);
                }
            }

            frag.set('center', new Point(
                rectangle.parent.strokeBounds.x +
                    rectangle.parent.strokeBounds.width / 2,
                rectangle.parent.strokeBounds.y +
                    rectangle.parent.strokeBounds.height / 2)
            );
        }
    }

    var processCellEvents = function(collection) {
        var tool = new Tool();

        onMouseDownHandler.collection = collection;
        onMouseMoveHandler.collection = collection;
        onMouseDragHandler.collection = collection;

        tool.onMouseDown = onMouseDownHandler;
        tool.onMouseMove = onMouseMoveHandler;
        tool.onMouseDrag = onMouseDragHandler;
    };

    var processOuterCellEvents = function(drawer, cellTableView) {
        cellTableView.listenTo(cellTableView, 'change:center_x',
            function(cellView, cellModel, val, oldval) {
                drawer.updateCenterX.call(cellModel, val-oldval, false);
        });

        cellTableView.listenTo(cellTableView, 'change:center_y',
            function(cellView, cellModel, val, oldval) {
                drawer.updateCenterY.call(cellModel, val-oldval, false);
        });

        cellTableView.listenTo(cellTableView, 'change:width',
            function(cellView, cellModel, val, oldval) {
                drawer.updateRWidth.call(cellModel, val, false);
        });

        cellTableView.listenTo(cellTableView, 'change:height',
            function(cellView, cellModel, val, oldval) {
                drawer.updateRHeight.call(cellModel, val, false);
        });
    };

    var updateCellLinks = function (cell, link) {
        //изменяем параметры связанности ячейки
        var r_link = this.find(
            function(cell) {
                return cell.get('idx') == parseInt(link[0]);
            }),
            cell_text_params = r_link.get('text_params');

        if (r_link) {
            var all_links = cell_text_params.rows.length;
            cell_text_params.rows.push('');
            cell_text_params.rows_offset.push(0);
            link[1] = all_links++;
            cell.set('text_params', cell_text_params);
        } else {
            //возможно что ссылка на родительский элемент теряется
            //и надо продолжать загрузку редактора
            link[0] = -1;
            link[1] = 0;
        }
    };

    //private
    function createFieldLabel(field, id) {
        /** создание метки в левом верхнем углу поля
         *  добавляем номер параметра слева от каждого поля
         *  @param field {Object} - объект ячейки редактора
         *  @param id    {Number} - номер метки
         */
        var cell_width, cell_height, cell_center, cell_group, id_text;

        console.log("CREATE LABEL-"+id);
        cell_width = field.width;
        cell_height = field.height;
        cell_center = field.center;
        cell_group = field.group_;

        id_text = new PointText(
                  new Point(cell_center.x - cell_width / 2 - 14,
                            cell_center.y + cell_height / 2 - 2));

        id_text.characterStyle = {
            font: fieldLabelDefaultFont, 
            fontSize: fieldLabelDefaultSize
        };

        id_text.content = id;

        cell_group.addChild(id_text);
        
        field.id_text = id_text;
        field.group_ = cell_group;
        field.idx = id;

        return field;
    }

    function updateLabel() {
        //обновляем метку
        var label = this.get('id_text'),
            width = this.get('width'),
            height = this.get('height'),
            center = this.get('center'),
            group = this.get('group_');

        label.remove();
        var field = createFieldLabel({
            "width": width, 'height': height,
            'center': center, 'group_': group},
            this.get('idx'));

        this.set('id_text', field.id_text);
    }

    //private
    function createRectangle(point, width, height, borderColor, config, innerBoundColor) {
        /**  Cоздание простого прямоугольного поля (ячейки)
        *  @point       {Object} - точка где будет расположен центр прямоугольника
        *  @width       {Number} - ширина прямоугольника в пикселях
        *  @height      {Number} - высота прямоугольника в пикселях
        *  @borderColor {String} - цвет окантовки (в hex - формате)
        *  @return      {Object} - объект прямоугольника
        */
        var xndiff = point.x - width /2, xpdiff = point.x + width /2,
            yndiff = point.y - height/2, ypdiff = point.y + height/2,

            line1 = new Path.Line(
                        new Point(xndiff, yndiff),
                        new Point(xpdiff, yndiff)),
            line2 = new Path.Line(
                        new Point(xpdiff, yndiff),
                        new Point(xpdiff, ypdiff)),
            line3 = new Path.Line(
                        new Point(xndiff, ypdiff),
                        new Point(xpdiff, ypdiff)),
            line4 = new Path.Line(
                        new Point(xndiff, yndiff),
                        new Point(xndiff, ypdiff));

        var initialConfig = config || {},
            group_ = new Group();
        //рисуем линии
        var unselect_color = borderColor || '#00ff00',
            lines = [line1, line2, line3, line4], ln, lns = [];

        for (var l = 0; l < lines.length; l++) {
            ln = lines[l];
            ln.strokeColor = unselect_color;
            ln.strokeWidth = 2;
            lns.push(ln);
            group_.addChild(ln);
        }
        //рисуем окантовку
        var boundedRectangle = new Path.Rectangle(
            group_.strokeBounds.x + 4,
            group_.strokeBounds.y + 4,
            width - 6, height - 6
        );

        boundedRectangle.fillColor = new RgbColor(255, 255, 255, 0);
        boundedRectangle.strokeColor = innerBoundColor || '#ff00ff';
        group_.addChild(boundedRectangle);

        return {
            'width': width,
            'height': height,
            'center': point,
            'group_': group_,
            'boundedRectangle': boundedRectangle,
            'lines': lns,
            'unselectColor': unselect_color
        };
    }

    function createSegmentRectangle(point, width, height, id) {
        /** Создание главного объекта редактора - ячейки высотой height и шириной width
         *  которую можно заполнить текстом
         *  и маркировки с указанием порядкового номера ячейки (id)
         */
        if (typeof(point) == "string") {
            //TODO - доделать вариант передачи поинта как строки
        }
        var field = createRectangle(point, width, height, '#ee2a33');
        //создание лейбла
        createFieldLabel(field, id+1);
        return field;
    }

    function deselectFunc(obj, editMode) {
        /**
         *  Снимает выделение с выбранного объекта (ячейки, поля)
         *  @obj        {Object}  - объект (поле) с которого нужно снять выделение
         *  @editMode   {Boolean} - режим работы редактора 
         */
         var singleSelect = function(obj) {
             var transparent_color = new RgbColor(255, 255, 255, 0),
                 lines = obj.get('lines'), edge,
                 activeColor;

             activeColor = editMode ? obj.get('unselect_color'): transparent_color;

             if (lines) {
                 //проходимся по всем ребрам прямоугольника и меняем цвет
                 for (var l = 0; l < lines.length; l++) {
                     edge = lines[l];
                     edge.strokeColor = activeColor;
                 }

                 var bound = obj.get('boundedRectangle');
                 bound.strokeColor = transparent_color;

                 var fieldMark = obj.get('id_text');
                 if (fieldMark) {
                     fieldMark.strokeColor = activeColor;
                     fieldMark.fillColor = activeColor;
                 }
             }
         };

         if (obj instanceof Backbone.Collection) {

             obj.each(function(item) {
                 singleSelect(item)
             });

         } else {
             singleSelect(obj);
         }
    }

    function createText(renderingMode, textString,
                        fontType, fontSize, width, height,
                        textPosition, appendTo,
                        printOffsetLeft, printOffsetTop, offsetType) {

        var calculateOffset = function(paramVal, offsetVal) {
            var recalcValue = paramVal;

            if (offsetType == "relative") {

                if (offsetVal < 0) {
                    recalcValue = paramVal * (-1) * offsetVal;
                } else {
                    recalcValue = paramVal * (1 / offsetVal);
                }

            } else if (offsetType == "absolute") {
                recalcValue = paramVal + offsetVal;
            }

            return recalcValue;
        };


        if (renderingMode == "DIVS") {

            var container = $(appendTo),
                text_block_div = '<div class=\"text_field\"></div>',
                fs = fontSize || fieldLabelDefaultSize,
                ft = fontType || fieldLabelDefaultFont;

            var txt_div = $(text_block_div);

            container.append(txt_div);
            txt_div.html(textString);

            var style = {
                fontFamily  : ft,
                fontSize    : fs + "pt",
                position    : 'absolute',
                left        : calculateOffset(textPosition.x - width / 2, printOffsetLeft),
                top         : calculateOffset(textPosition.y + height / 2 - parseFloat(helpers.to_pUnit(parseInt(fs) * 1/72 * 24.5)), printOffsetTop),
                width       : width, height: height,
                display     : 'block'
            };
            txt_div.css(style);
        } else {
            //TODO - расширить дляальтернативных режимов рендеринга
        }
    }   

    function destroyText(value) {
        value.remove();
    }

    function clearText(container) {
        var text_fields = container.find('.text_field');
        text_fields.remove();
    }

    function updateCenterX(delta, calculate, absolute) {
        /** изменяем левую границу ячейки
         *  @param delta - значение в пикселях смещения левой границы объекта
         *  @param absolute - если true значит delta - это абсолютная координата центра по оси X
         */
        var old_rec = new this.constructor,
            width = this.get('width'),
            center = this.get('center'),
            point, new_x_pos;

        old_rec.set('width', this.get('width'), {silent:true});
        old_rec.set('height', this.get('height'), {silent:true});
        old_rec.set('center', this.get('center'), {silent:true});
        //переносим центр в новую точку
        if (!absolute) {
            new_x_pos = center.x + delta
        } else {
            new_x_pos = delta;
            delta = delta - center.x;
        }

        point = new Point(new_x_pos, center.y);

        if (!calculate) {
            this.set('center', point, {silent:true});
            updateRectangleGeometry(this, old_rec, {delta_x: delta});
        } else {
            return {
                delta_x: delta,
                set: {center_x: point}, 
                cell_copy: old_rec};
        }
    }

    function updateCenterY(delta, calculate, absolute) {
        /** изменяем вурхнюю границу ячейки
         *  @param delta - значение в пикселях смещения верхней границы объекта
         */
        var old_rec = new this.constructor,
            height = this.get('height'),
            center = this.get('center'),
            point, new_pos_y;
            
        old_rec.set('width', this.get('width'), {silent:true});
        old_rec.set('height', this.get('height'), {silent:true});
        old_rec.set('center', this.get('center'), {silent:true});

        if (!absolute) {
            new_pos_y = center.y + delta;
        } else {
            new_pos_y = delta;
            delta = delta - center.y;
        }

        point = new Point(center.x, new_pos_y);

        if (!calculate) {
            this.set('center', point, {silent:true});
            updateRectangleGeometry(this, old_rec, {delta_y: delta});
        } else {
            return {
                delta_y: delta,
                set: {center_y: point},
                cell_copy: old_rec};
        }
    }

    function updateRHeight(height_, calculate) {
        
        var old_rec = new this.constructor,
            offset = height_ - old_rec.get('height'),
            point;

        old_rec.set('width', this.get('width'), {silent:true});
        old_rec.set('height', this.get('height'), {silent:true});
        old_rec.set('center', this.get('center'), {silent:true});

        point = new Point(
                    old_rec.get('center').x,
                    old_rec.get('center').y + offset / 2);

        if (!calculate) {
            this.set('height', height_, {silent:true});
            this.set('center', point, {silent:true});
            updateRectangleGeometry(this, old_rec, {height: true});
        } else {
            return {
                height: height_,
                cell_copy: old_rec,
                set: {
                    center_y: point
                }
            };
        }

    }

    function updateRWidth(width_, calculate) {

        var old_rec = new this.constructor,
            offset = width_ - old_rec.get('width'),
            point;

        old_rec.set('width', this.get('width'), {silent:true});
        old_rec.set('height', this.get('height'), {silent:true});
        old_rec.set('center', this.get('center'), {silent:true});
        //при изменениях ширины меняем центр ячейки
        point = new Point(
                    old_rec.get('center').x + offset/2,
                    old_rec.get('center').y);

        if (!calculate) {
            this.set('width', width_, {silent:true});
            this.set('center', point, {silent:true});
            updateRectangleGeometry(this, old_rec, {width: true});
        } else {
            return {
                width: width_,
                cell_copy: old_rec,
                set: {
                    center_x: point
                }
            };
        }
    }

    return {
        selectionModel: selection_model,
        drawCell: drawCell,
        eraseCell: eraseCell,
        deselectFunc: deselectFunc,
        createFieldLabel: createFieldLabel,
        updateLabel: updateLabel,
        createSegmentRectangle: createSegmentRectangle,
        processCellEvents: processCellEvents,
        processOuterCellEvents: processOuterCellEvents,
        createText: createText,
        destroyText: destroyText,
        clearText: clearText,
        updateRWidth: updateRWidth,
        updateRHeight: updateRHeight,
        updateCenterX: updateCenterX,
        updateCenterY: updateCenterY,
        updateRectangleGeometry: updateRectangleGeometry
    }
});
