
define('drawing/layer_manager', ['jquery', 'paper'], function($, Paper) {

    var Manager = {
        objects: [],
        selector: null,
        setupPanel: null,
        init: function() {
            var me = this;
            this.defineControls();
            //инициализируем опции выбора объектов
            var oltag = '<option>', ortag = '</option>', optstr = '';
            for (var o = 0; o < me.objects_names.length; o++) {
                optstr += [oltag, me.objects_names[o], ortag].join("");
            }
            this.selector.html(optstr);
            this.populateObjects();
        }
    };

    var LayerManager = function(project, manipulators) {
        var me = this;

        this.autoActivate = true;
        this.project = project;
        this.objects_names = [
            'Поля', 'Блоки'
        ];

        this.manipulators = manipulators;
        this.defineControls = function() {
            //определение контроллов для управления объектами слоя
            var wrapper = $("#select_from_layers");
            this.selector = wrapper.find('select');
            this.activeButton = wrapper.find('button');
            if (this.autoActivate) {
                this.activeButton.css('display', 'none');
                this.selector.on('change', this.activate);
            }
            else {
                this.activeButton.css('display', 'block');
                this.selector.unbind('change', this.activate);
            }
        };
        this.populateObjects = function() {
            this.objects = [];
            var layers = [
                new Layer(),
                new Layer()
            ];
            for (var o = 0; o < layers.length; o++) {
                this.objects.push(layers[o]);
            }
            //ассоциируем слои и манипуляторы
            for (var m = 0; m < this.manipulators.length; m++) {
                var mn = this.manipulators[m];
                mn.layer = this.getLayerById(m);
            }
        };
        this.activate = function(e) {
            var layer_name = me.selector.val(),
                name_index = me.objects_names.indexOf(layer_name),
                object_ = me.objects[name_index];

            me.setActiveLayer(object_);
            //TODO - надо сделать слот для события
        };
        this.setActiveLayer = function(object) {
            //устанавливаем текущий активный слой
            var index_of = this.objects.indexOf(object);
            console.log("SET ACTIVE LAYER -> " + index_of);
            this.deactivateAllLayers();
            object.activate();
            object.visible = true;
        };
        this.deactivateAllLayers = function() {
            //делаем невидимыми все слои
            for (var l=0; l < me.objects.length; l++) {
                var layer = me.objects[l];
                layer.visible = false;
            }
        };
        this.getLayerById = function(id) {
            return me.objects[id];
        };
        this.onLayerMouseDown = function() {
            var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
                manipulator = this.manipulators[active_layer_idx];
            return manipulator.onMouseDown;
        };
        this.onLayerMouseMove = function() {
            var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
                manipulator = this.manipulators[active_layer_idx];
            return manipulator.onMouseMove;
        };
        this.onLayerMouseDrag = function() {
            var active_layer_idx = this.objects.indexOf(this.project.activeLayer),
                manipulator = this.manipulators[active_layer_idx];
            return manipulator.onMouseDrag;
        };
        this.onTemplateSave = function(request_params) {
            /*
             Обработка параметров шаблона перед сохранением на диск
             *  @request_params {Object} - параметры шаблона
             */
            debugger;
            var layer_configurations = [];
            for (var m = 0; m < me.manipulators.length; m++) {
                layer_configurations.push(
                    me.manipulators[m].getObjectsConfiguration()
                );
            }

            $.extend(request_params, {
                "layers_configurations": $.toJSON(me.manipulators)
            });
        }

    };

    LayerManager.prototype = Manager;
    return LayerManager;
});
