define('drawing/zone_manipulator', ['jquery', 'drawing/manipulator'], function($, Manipulator) {

    var ZoneManipulator = function(project, layer) {
        /**  Зональный манипулятор
         Зона - область в редакторе которая может определять
         местоположение обычных полей (fields). С их помощью
         можно управлять положением групп полей перемещением
         их между различными зонами.
         *  @constructor
         *  @project - объект проект (paper.js)
         *  @layer - объект слоя (paper.js)
         */

        this.project = project;
        this.layer = layer;
        this.viewOnStartup = false;
        this.objects = [];
        this.fn = this.prototype;
        var me = this;
        this.defineElements = function() {
            this.optionShowElement = $('#show_zones_setup_check');
            this.setupElement = $('#zones_setup');
            //элемент представления хранящий информацию о объектах данного слоя
            this.itemsInfoView = this.setupElement.find('table');
            this.setupElement.css('minHeight', '100px');
            this.toolbar = this.setupElement.find('.toolbar');
            this.addBtn = this.toolbar.find('.add_button');
            this.delBtn = this.toolbar.find('.del_button');
            this.addBtn.on('click', this.addZone);
            this.delBtn.on('click', this.delZone);
        };
        this.getParentGroup = function(obj) {
            var om = this.objects;
            for (var n = 0; n < om.length; n++) {
                if (om[n].group_ === obj) {
                    return [om[n], n, null];
                }
            }
            return [null, 0, null];
        };
        this.onMouseDrag = function(event) {
            console.log("ZONE MOVED!");
            var object = event.item;
            if (me.isObjectSelected) {
                //            console.log("MOVE OBJECT!");
                //            debugger;
                me.moveObject(me.selected, event.delta);
            }
            else if (me.isBorderSelected) {
                console.log("MOVE BORDER!");
                me.moveBorder(me.selected, event.delta);
            }
        };
        this.onMouseDown = function(event) {
            console.log("ZONE SELECT!");
            var object = event.item;
            me.getSelectedObject(event.point);
        };
        this.getObjectById = function(object_id) {
            return me.objects[object_id];
        };
        this.deselectAllObjects = function() {
            //снимаем выделение со всех объектов
            me.project.activeLayer.selected = false;
            for (var n =0 ; n < this.objects.length; n++) {
                //FIXME - починить данный метод (добавили еще один параметр обязятельный)
                deselectFunc(this.objects[n]);
            }
        };
        this.isBoundSelected = function(object, select_item_id, all_set) {
            //проверка выбраны ли границы объекта
            var set = null, scnt=0, dcnt=0,
                lines = [
                    object.lines[0].id,
                    object.lines[1].id,
                    object.lines[2].id,
                    object.lines[3].id
                ],
                select_in_lines = lines.indexOf(select_item_id);

            if (select_in_lines != -1 && all_set) {
                set = true;
            }
            if (select_in_lines == -1 && !all_set) {
                set = true;
            }

            return set
        };

        this.getSelectedObject = function(hit_point) {
            /** Определение объекта выбора путем анализа результатов коллизий
             *
             */
            var me = this;
            if (this.layer !== this.project.activeLayer) {
                return;
            }
            me.deselectAllObjects();
            var hit_res = me.project.hitTest(hit_point, hitOptions);
            if (!hit_res) {
                return;
            }
            var type = hit_res.type;
            if (hit_res && hit_res.item && !editMode) {

                var selected_item = hit_res.item;

                if (type == 'stroke') {
                    selected_item.strokeColor = "#00ff00";
                    var rec = me.getParentGroup(selected_item.parent),
                        oid = rec[1], obj = rec[0],
                        object = me.getObjectById(oid);
                    console.log(rec);
                    //пробуем определить группу к которой принадлежит выбранный элемент
                    if (obj != null &&
                        me.isBoundSelected(object, selected_item.id, false)) {
                        //выбран внутренний прямоугольник
                        me.isBorderSelected = false;
                        me.isObjectSelected = true;
                    }
                    else if (obj != null &&
                        me.isBoundSelected(object, selected_item.id, true)) {
                        //выбран внешний край
                        me.isBorderSelected = true;
                        me.isObjectSelected = false;
                    }
                }
                else if (type == 'fill') {
                    console.log("SELECT FILL");
                    me.isBorderSelected = false;
                    me.isObjectSelected = true;
                }
                console.log("SELECT " + type);
                selected_item.selected = true;
                this.selected = selected_item;
            }

        };

        this.addZone = function(btn, width, height, point) {

            var zone_layer = me.layer;
            layer_mgr.setActiveLayer(zone_layer);

            var zwidth = width || 20, zheight = height || 20,
                zpoint = point || new Point(zwidth / 2, zheight / 2),
                zone = createRectangle(zpoint, zwidth, zheight, '#0066ff',
                    {showTooltip: false, manager: me.objects}
                );

            me.objects.push(zone);
            me.createObjectInfo();
        };

        this.delZone = function() {
            var zone = null;
            me.objects.remove(zone);
        };

        this.getModelFields = function() {
            return ['name', 'position', 'childs'];
        };

        this.getObjectInfo = function(id) {
            var fields = me.getModelFields(),
                row = this.itemsInfoView.find('tr').eq(id),
                columns = row.find('td'), info = {}, field_counter = 0;

            columns.each(function(c) {
                info[fields[++field_counter]] = c.html();
            });
            return info;
        };

        this.createObjectInfo = function() {
            debugger;
            var fields = me.getModelFields(),
                row = this.itemsInfoView.append('<tr></tr>').find('tr').eq(-1);

            for (var f = 0; f < fields.length; f++) {
                var column = row.append('<td></td>').find('td').eq(-1);
                column.html('<input name="' + fields[f] + '" />');
            }
        };

        this.getObjectsConfiguration = function() {
            var rows = this.itemsInfoView.find('tr'),
                config = [];
            rows.each(function(r) {
                var columns = r.children(),
                    fields = me.getModelFields(),
                    row_info = {}, field_counter = 0;
                columns.each(function(c) {
                    row_info[fields[++field_counter]] = c.val();
                    config.push(row_info);
                });
            });
            return config;
        };

        this.fixToZone = function() {
            //установка группы элементов в определенную зону

        };
        this.updatePositionData = function(object_id) {
            //меняем информацию по расположению зон
        };
        return this;
    };

    ZoneManipulator.prototype = Manipulator;

    return ZoneManipulator;
});
