/*шаблон Модуль*/
/**
 * Модуль Редактор
 *  методы: loadData, getLayerParams, getParamValues, setPrinterParams, init
 */

define('editor', ['jquery', 'helpers',
                  'drawing/cell_manipulator', 'paper',
                  'drawing/drawer',
                  'cells/cellmanager',
                  'hyphenation/hyphenator'
                 ],
function ($, helpers, CellManipulator, Paper, Drawer, CellManager, Hyphenator) {

    var print_mode = 0,
        edit_mode = 1,
        //режим печати текста (можно печать текст как картинку на канвасе, так и просто как DIVки)
        text_render_modes = ['CANVAS', 'DIVS'];

    function getTooltip() {
        //возвращаем вьюху подсказки (тултипа)
    }

    function getLayerParams(field_idx, param_name, map) {

        var param_order = {
            'center_x' 	: {"idx": 0, "default_value": 70 },
            'center_y' 	: {"idx": 1, "default_value": (70 + field_idx * 20 + 1)},
            'width' 	: {"idx": 2, "default_value": 20 },
            'height' 	: {"idx": 3, "default_value": 20 },
            'print_off' : {"idx": 4, "default_value": 0  },
            'linked' 	: {"idx": 5, "default_value": -1 },
            'font_type' : {"idx": 6, "default_value": "Arial" },
            'font_size' : {"idx": 7, "default_value": "12" }
        };

        var field_params = map[field_idx],
            map_idx = param_order[param_name] ?
            param_order[param_name]["idx"] : null,
            val = null;

        if (map_idx >= 0 && field_idx < map.length
            && field_params.length > map_idx) {
            val = field_params[map_idx];
        } else if (map_idx >= 0) {
            val = param_order[param_name]["default_value"];
        }

        return val;
    }

    function loadData(var_args) {
        var that = this;
        //функция вставляет данные по параметрам отчета в соответствующие поля
        var hyphen = new RegExp(String.fromCharCode(0x00ad),'g');
        // console.log("SERVER DATA LOAD=" + (var_args && var_args == "no_server_data"));
        var hyphen_mode = false;

        if (var_args && var_args.length==1 && var_args[0] == "hid_hyp") {
            // console.log("LOAD AFTER HYPHENATION");
            hyphen_mode = true;
        }

        //TODO - здесь предполагается что мы должны отложить заполнение
        //данных пока не будут получены данные с сервера
        //т.е loadData будет повторно вызвана из функции getParamsValues
        if ((var_args && var_args == "no_server_data") ||
            (var_args != "fill_fields" && getParamsValues(that))) return -1;
        var val_array = [];

        //заполняем поля текстом. При этом данные должны быть переданы
        //в аргументе var_args или если var_args не задан то данные
        //запрашиваются у сервера

        if (var_args == undefined || hyphen_mode || var_args == 'fill_fields') {
            var values = $(".hidden_data"),
                description = $(".hidden_data1"),
                start = 0,
                end = that.number_of_params;
            //включаем диапазон игнорирования некоторых полей для переноса
            if (hyphen_mode) {
                start = that.cell_rng_start;
                end = that.cell_rng_end;
            }

            values.each(function(e) {
                if ((e < that.number_of_params && !hyphen_mode)
                    || (hyphen_mode && (e<start || e > end))) {
                    // console.log("VAL=" + $(this).html());
                    val_array.push($(this).html());
                }
            });

            $(".description").each(function(e) {
                $(this).html(description.eq(e-1).html());
            });
        }
        else {
            //предполагаем что в аргументе переданы данные для заполнения полей
            if (!hyphen_mode) {
                val_array = var_args;
            }
        }

        //удаляем все текстовые метки если используется режим вывода текста через HTML
        if (that.render_mode=="DIVS") {
            $(".text_field").remove();
        }

        //TODO - реализовать хранение ячеек как моделей в Backbone.js
        var recs = CellManager.getCollection();

        for (var i=0; i < recs.length; i++) {

            if (recs[i].print == true) {
                // if (val_array.length>0) {
                //ПРИ ОБНОВЛЕНИИ ПОЛЕЙ ПОСЛЕ ПЕРЕНОСА НЕ НУЖНО
                //УДАЛЯТЬ ПОЛЯ КОТОРЫЕ НЕ БЫЛИ ЗАТРОНУТЫ ПЕРЕНОСОМ
                if (recs[i].value) {
                    recs[i].value.remove();
                }

                var text = new PointText(
                    new Point(recs[i].center.x-recs[i].width/2+2,
                        recs[i].center.y+recs[i].height/2-2));

                var cur_font = $("#font_selector").val();
                var cur_fsize = parseInt($("#selector_fontsize").val());
                //смотрим параметры текста
                var saved_text = '';

                //замена текста из системы хранится в первом индексе
                if (recs[i].text_params.rows.length)
                    saved_text = recs[i].text_params.rows[0];
                //в приоритете данные которые поступают из родительских полей и разбиение по строкам
                if (recs[i].linked_id[0]!=-1) {
                    saved_text = recs[recs[i].linked_id[0]].text_params.rows[recs[i].linked_id[1]];
                }
                text.content = saved_text || val_array[i] || '';
                //замечен баг если в тексте использован экранирующий слеш
                text.content = text.content.replace(/\\\\/g,'\\');
                text.characterStyle = {
                    font: recs[i].text_params.font_type || cur_font,
                    fontSize: recs[i].text_params.font_size|| cur_fsize
                };
                //нужно для всех полей убрать символы мягкого переноса если они есть
                if (hyphen_mode) {
                    text.content = text.content.replace(hyphen,'');
                }
                recs[i].value = text; //делаем ссылку на объект текста (надо для последующего удаления объекта при переключении режимов)
                if (hyphenation_on == 0) {
                    Hyphenator.singleHyphenation(i);
                }

                if (!recs[i].hyphenate) {
                    recs[i].value = text;
                } else {
                    recs[i].value.content = recs[i].text_params.rows[0].replace(hyphen,'');
                }

                that.renderText(recs[i]);
                // if (i<100)
                //  	console.log("SET TO FIELD->#" + i + "->" + text.content + ", FROM:" + saved_text + ", "+ val_array[i] + ", " + ((var_args && var_args[i]) || ''));
            }

        }

        return 0;
    }
    
    //private
    function getParamsValues(editor) {
        /*запрос параметров для заполнения полей (ячеек)*/
        // console.log("GET DATA FROM SERVER");
        //функция заполняет данные для отчета при перезагрузке страницы
        var param_vals = $(".hidden_data"), that = editor;

        //если параметры уже есть в шаблоне не будем делать
        //запрос к серверу
        if (param_vals.length > 0) {
            return 0;
        }

        $.ajax({
            url: editor.getActions().get_params_url,
            dataType: 'json',
            type: 'POST',
            data : {
                pack_url: editor.getActions().provider_url,
                id_field: $("#id_field_val").val(),
                add_vals: $("#add_vals_field").val()
            },
            success: function(data, statusText) {

                if (data['success'] === false) {
                    alert("При загрузке данных возникли проблемы!");
                    return ;
                }
                var html='';
                //data - представляем из себя 2 списка один
                //1ый - список значений полей
                //2ой - список описания полей
                //получаем данные из полей
                //если data передается в виде единого списка то предполагаем что описания полей отсутствует
                //а есть только значения
                var values = {};
                var descriptions = {};

                if (data.length == 1) {
                    values = data;
                }
                else if (data.length == 2) {
                    values = data[0];
                    descriptions = data[1];
                }

                for (var p in values) {
                    html+='<div id=\"paramval_' + p + ' \" style=\"{display:none;}\" class=\"hidden_data\">' +
                        data[0][p] + '</div>'
                }
                for (var p in descriptions) {
                    html+='<div id=\"paramdesc_' + p + ' \" style=\"{display:none;}\" class=\"hidden_data1\">' +
                        data[1][p] + '</div>'
                }

                $("#report_data").html(html);
                // console.log("DATA FROM SERVER LOADED");

                // //заполняем поля принятыми от сервера данными
                // var data_loaded = loadData();
                // console.log("load status = " + data_loaded);
                //обновляем состояние канваса и заполняем данными поля
                changePrintStyle("fill_fields", that);
            },
            failure: function(data) {
                alert("Ошибка при загрузке данных для шаблона!");
            }
        });
        return 1;
    }
        
    function setDefault() {
        var that = this;
        that.coord_map = [];
        for (var crd in pre_coord_map) {
            that.coord_map.push(pre_coord_map[crd]);
        }
        //добавляем поля на канвас
        CellManager.addField(this);
        //заполняем таблицу с координатами
        CellManager.getCoords(this);
        //загружаем данные

        that.loadData();
    }
    //private
    function setPrinterParams(editor) {
        //выводим окно печати предварительно применив смещения полей
        var ths = $("#set_page_glparams"),
            left_offset = 0, top_offset = 0,
            nscale_x = 1, nscale_y = 1,
            that = editor;

        if (ths.attr("checked") == "checked") {
            nscale_x = fix_x;
            nscale_y = fix_y;

            left_offset = offset_x;
            top_offset = offset_y;

            var image_offset_x = 0;//parseFloat($("#paperCanvas").css("backgroundPosition").split(" ")[0]);
            var image_offset_y = 0; parseFloat($("#paperCanvas").css("backgroundPosition").split(" ")[1]);

            $("#paperCanvas").css("backgroundRepeat","no-repeat");
            that.container.css("backgroundRepeat","no-repeat");
            var background_pos = helpers.to_pUnit((image_offset_x + left_offset).toString())
                + "px " + helpers.to_pUnit((image_offset_y + top_offset).toString()) + "px";
            $("#paperCanvas").css("backgroundPosition", background_pos);
            that.container.css("backgroundPosition", background_pos);
        }
        else {
            nscale_x = 1/fix_x;
            nscale_y = 1/fix_y;

            left_offset = (-1)*offset_x;
            top_offset = (-1)*offset_y;
            $("#paperCanvas").css("backgroundPosition", "0px 0px");
            that.container.css("backgroundPosition", "0px 0px");
        }

        if (left_offset) {
            Drawer.bulkUpdateParams(
                [0, recs.length], "ol", left_offset.toString());
        }
        if (top_offset) {
            Drawer.bulkUpdateParams(
                [0, recs.length], "ot", top_offset.toString());
        }

        if (nscale_x != 1 || nscale_y != 1) {
            Drawer.bulkUpdateParams([0, recs.length], "sw", nscale_x);
            Drawer.bulkUpdateParams([0, recs.length], "sl", nscale_x);
            Drawer.bulkUpdateParams([0, recs.length], "sh", nscale_y);
            Drawer.bulkUpdateParams([0, recs.length], "st", nscale_y);
        }

        that.offset_x = parseFloat($("#set_left_offset").val());
        that.offset_y = parseFloat($("#set_top_offset").val());
        that.fix_x = parseFloat($("#set_fix_x").val());
        that.fix_y = parseFloat($("#set_fix_y").val());

    }

    function onBlurEvent() {
        var ths = $(this);
        ths.remove();
    }

    function clearPages() {
        this.cpages = [];
        this.cpage = 0;
    }

    function initEditor() {
        //инсталлируем библиотеку рисования Paper.js
        paper.install(window);
        paper.setup("paperCanvas");

        var that = this;
        //настраиваем размер канваса в зависимости от реального разрешения экрана
        this.dpi = helpers.getDpi();
        //количество пискелей в одном миллиметре
        var pixel_size = this.dpi / 25.4,
            cvas = $("#paperCanvas"),
            sub_picture_size = $("#blank_size").html(),
            blank_size = this.layer.canvas_default_size,
            blank_width = parseFloat(blank_size.split(',')[0]),
            blank_height = parseFloat(blank_size.split(',')[1]),
            blank_pixel_width = blank_width * pixel_size,
            blank_pixel_height = blank_height * pixel_size,
            sub_picture_width = parseFloat(sub_picture_size.split(',')[0]),
            sub_picture_height = parseFloat(sub_picture_size.split(',')[1]),
            sub_pixel_width = sub_picture_width * pixel_size,
            sub_pixel_height = sub_picture_height * pixel_size;

        //задание размера для канваса
        //используем настройки канваса по умолчанию
        cvas.attr("width", Math.floor(
            sub_pixel_width.toFixed(2)).toString() + "px");
        cvas.attr("height", Math.floor(
            sub_pixel_height.toFixed(2)).toString() + "px");

        var sub_picture_lft = (blank_pixel_width - sub_pixel_width)/2;

        //позиционируем подложку так чтобы она располагалась по середине

        this.container.css("width",
            Math.floor(blank_pixel_width.toFixed(2)).toString() + "px");
        this.container.css("height",
            Math.floor(blank_pixel_height.toFixed(2)).toString() + "px");

        if (sub_picture_lft > 0) {
            var left_val = Math.floor(sub_picture_lft.toFixed(2)).toString() + "px";
            cvas.css("left", 0);
            editor.container.css('left', left_val);
        }

        $("#print2side").css("top",
            Math.floor(sub_pixel_height.toFixed(2) + 20.0).toString() + "px");


        var field_mgr = new CellManipulator(project, null, this);
        field_mgr.init();
    //        var zones_mgr = new ZoneManipulator(project);
    //        zones_mgr.init();
    //        layer_mgr = new LayerManager(project,
    //            [field_mgr, zones_mgr]
    //        );
    //        layer_mgr.init();

        //добавляем поля на канвас
        CellManager.addField(this);
        //заполняем таблицу с координатами
        CellManager.getCoords(this);
        //загружаем данные
        //здесь может быть проведен запрос к серверу на загрузку данных
        //для полей
        var data_exist = that.loadData();

        loadAllTemplates(that);
        setPrinterParams(that);

        if (data_exist != -1) {
        //если данные еще не загружены то обновляем состояние
        //канваса после загрузки
            changePrintStyle(null, that);
        }

    }

    //TODO - перенести во вьюху координат
    function updateCoords(rec_id) {
        //отображение координат в реальном времени при перемещении элемента или изменении размеров
        var id = parseInt(rec_id) + 1;

        //TODO - при большом количестве ячеек здесь наблюдается просадка по производительности
        var coord_info_row = $("#cinfo" + id);

        coord_info_row.children('td').slice(1,2).children().val(
            helpers.to_hUnit(recs[rec_id].center.x.toFixed(2) - recs[rec_id].width / 2));
        coord_info_row.children('td').slice(2,3).children().val(
            helpers.to_hUnit(recs[rec_id].center.y.toFixed(2) - recs[rec_id].height / 2));
        coord_info_row.children('td').slice(3,4).children().val(
            helpers.to_hUnit(recs[rec_id].width.toFixed(2)));
        coord_info_row.children('td').slice(4,5).children().val(
            helpers.to_hUnit(recs[rec_id].height.toFixed(2)));
    }

    function checkPrint() {
        this.print();
    }

    var editor = {
        mode : print_mode,
        //главная вьюха редактора
        container: $("#blank_container"),
        //текущий режим рендеринга текста(по умолчанию как div)
        render_mode : text_render_modes[1],
        setMode : function() {
            /*установка режима редактирования или печати*/
            var mode = $("#set_editor_mode").attr("checked")
                == "checked" ? print_mode : edit_mode;

            this.mode = mode;
        },
        getMode : function() {
            return this.mode;
        },
        print : function() {
            //обработка нажатия на кнопку печать
            //TODO - сделать методом соответствующего контроллера
            if (this.mode == print_mode) {
                window.print();
            }
            else {
                //TODO - заменить на вьюху ошибки
                alert("Для печати бланка отключите режим \"Редактирование\"!" +
                    "\r\nДля этого включите флажок \"Печать/редактирование\"");
            }
        },
        setOffsets: function(ths) {
            //Установка отступов для подгонки под настройки принтера
            // console.log(ths);
            var el = $(ths);
            if (this.mode == print_mode) {
                alert("Установка отступов невозможна в режиме печати!\r\n" +
                    "Включите режим \"Редактирование\"");
                //инвертируем значение
                if (el.attr("checked") == "checked") el.removeAttr("checked");
                else el.attr("checked","checked");

                return false;
            }
            setPrinterParams();
        },
        renderText: function(rc) {
            //метод рендеринга текста ячеек полей редактор
            if (this.render_mode == 'DIVS') {
                console.log(rc.text_params.font_type + "-" + rc.text_params.font_size);
                var blank_container = this.container,
                    text_block_div = '<div class=\"text_field\"></div>';
                var txt_div = $(text_block_div);

                blank_container.append(txt_div);
                txt_div.html(rc.value.content);
                var style = {
                    fontFamily : rc.text_params.font_type,
                    fontSize : rc.text_params.font_size+"pt",
                    position: 'absolute',
                    left: rc.center.x - rc.width/2,
                    top: rc.center.y - rc.height/2,
                    width: rc.width, height: rc.height,
                    display: 'block'
                };
                txt_div.css(style);
            }
        },
        clearText: function() {
            //метод очистки текста для полей на холсте
            if (this.render_mode == "DIVS") {
                var blank_container = this.container;
                var text_fields = blank_container.find('.text_field');
                text_fields.remove();
                $("#paperCanvas").css('display', 'block');
            }
        },
        getActions: function() {
            //адреса для Ajax запросов на сервер
            return {
                save_template: this.save_template_url,
                load_template: this.load_template_url,
                upload_blank:  this.upload_blank,
                get_params_url: this.get_params_url,
                provider_url: this.provider_url
            }
        },
        saveTextParams: saveTextParams,
        getLayerParams: getLayerParams,
        init: init
    };

    function setElements(main_view) {
        var that = this;
        //устанавливаем урл на сохранение шаблона
        that.setToForm();
        //устанавливаем режим редактора
        that.setMode();

        $("#report_data").html(main_view);
        $("#template_path").val($("#path_to_template").html());
        $("#tempfile").val($("#path_to_tempfile").html());
        $("#num_of_params").val($("#number_of_params").html());
        $("#pack_url").val($("#data_func_pack").html());
        $("#blank_size_val").val($("#blank_size").html());
        $("#id_field_val").val($("#id_field").html());
        $("#add_vals_field").val($("#add_vals").html());

        $("#set_fix_x").val(this.fix_x);
        $("#set_fix_y").val(this.fix_y);
        $("#selector_fontsize").val(this.font_size);
        $("#font_selector").val(this.font_type);
        $("#set_left_offset").val(this.offset_x);
        $("#set_top_offset").val(this.offset_y);

        //получим полный путь до бекграунда
        var full_path_to_background = "url(\"+"+staticUrl+"/" + background_img_url + "\")";
        //если путь при загрузке пустой раннее сохраненный бекграунд
        $("#paperCanvas").css("background-image", full_path_to_background);
        $("info").css("width", "0px");


        this.container = $("#blank_container");
        var blank = $("#paperCanvas");

    }

    var constructor_ = function(config) {

        //ниже идут настройки и переменные состояния
        //выбранный элемент(универсально)
        this.recs = [];
        this.selected_item = null;
        //выбранный сегмент
        this.segment = null;
        //выбранный прямоугольник
        this.rectangle = null;
        this.global_group = null;
        //текущая подсказка для элемента (для настройки переносов текста)
        this.tooltip = null;
        this.tooltip_edit = 0;
        //режим редактирования или печати
        this.editMode = 0;
        //разрешение монитора по умолчанию
        this.dpi = 96;
        //настройки для определения коллизий для Paper.js
        this.hitOptions = {
            segments: true,
            stroke: true,
            fill: true,
            tolerance: 5
        };
        //разрывы страниц (индекс строки которая
        //переносится на следующую страницу)
        this.cpages = [0];
        //индекс текущей страницы
        this.cpage = 0;

        $.extend(true, this, config);
        //public модули
        this.drawer = Drawer;
        //МЕТОДЫ РЕДАКТОРА
        this.renderPage = renderPage;
        this.loadData = loadData;
//        this.getActions = getActions;
        return this;
    };

    constructor_.prototype = editor;

    return constructor_
});