
define('hyphenation/hyphenator', ['jquery', 'paper', 'editor'],

    function($, Paper, Editor) {

        var hypsym = String.fromCharCode(0x00ad);

        function getHidBefore(text) {
            //функция возвращает массив в котором хранится значение
            //переносов для каждого символа
            var hyps, hyp_cnt;
            hyps = [];
            hyp_cnt = 0;

            for (var ch = 0; ch < text.length; ch++) {
                if (text[ch] == hypsym) {
                    hyp_cnt += 1;
                }
                else hyps.push(hyp_cnt);
            }

            return hyps;
        }

    function getBreakIndex(text, cell, fontType, fontSize) {

        //функция возвращает
        var oneCharacter, cwidth, cheight, chrCnt,
            cellWidth, cellHeight, symbolsInCell,
            chrDiff, hidCnt, shyIdx, breakIdx;

        //предполагается что используется моноширинный шрифт
        oneCharacter = new PointText(new Point(text.bounds.x, text.bounds.y));
        oneCharacter.characterStyle = {
            font: fontType,
            fontSize : fontSize
        };
        oneCharacter.content = "A";
        cwidth = oneCharacter.bounds.width;
        cheight = oneCharacter.bounds.height;
        oneCharacter.remove();

        cellWidth = cell.get('width');

        //ищем позицию в тексте где можно перенести текст
        chrCnt = Math.floor(text.bounds.width / cwidth);
        //возможное количество символов в ячейке
        //TODO - число 10 - эмпирическое значение (визуально отображаемое)
        symbolsInCell = Math.floor((cellWidth + 10) / cwidth);
        //считаем сколько символов не умещается в ячейке
        chrDiff = chrCnt - symbolsInCell;
        hidCnt = getHidBefore(text.content)[symbolsInCell - 1];
        shyIdx = text.content.slice(0, symbolsInCell + hidCnt).lastIndexOf(hypsym);
        breakIdx = -1;

        //приоритет остается за символом мягкого переноса
        if (shyIdx > 0 && chrDiff >= 0) breakIdx = shyIdx;
        else if (chrDiff >= 0) breakIdx = symbolsInCell-1 + hidCnt;

        //if (DEBUG) {
        // console.log("---DATA---"
        // 	+ "\r\n TEXT           -> " + text.content
        // 	+ "\r\n TEXT LEN       -> " + text.content.length
        // 	+ "\r\n FONT TYPE      -> " + oneCharacter.characterStyle.font
        // 	+ "\r\n FONT SIZE      -> " + oneCharacter.characterStyle.fontSize
        // 	+ "\r\n SYMBOL WIDTH   -> " + cwidth
        // 	+ "\r\n SYMBOL COUNT   -> " + symbolsInCell
        // 	+ "\r\n EXCEEDS SYMS   -> " + chrDiff
        // 	+ "\r\n HIDSYMS BEFORE -> " + hidCnt
        // 	+ "\r\n BREAK INDEX    -> " + breakIdx
        // 	+ "\r\n BREAKED TEXT   -> " + text.content.slice(0, breakIdx)
        // );
        //}

        return breakIdx;
    }

    function autoBreakText(text, rec, lines, lines_obj, mode) {
        //функция переноса по символам текста ДЛЯ МОНОШИРИННОГО ШРИФТА
        //получим характеристики одного символа текста для выбранного шрифта и размера
        //если текст не привязан к группе то удаляем его со слоя
        //mode - режим переноса, подразумевается пока 2 режима
        //1 - режим при котором для пробы размещения текста используется корневая несвязная ячейка
        //для все строковых переносов (если mode не определен то предполагается этот режим)
        //2 - режим в котором для пробы используется ячейка на которую делается перенос(этот режим надо задавать явно)
        var new_line = '', breakIdx, newText, breakResult;
        //удаляем текст если он принадлежит полю

        if (!(text.parent instanceof Group)) text.remove();
        //получаем граничный индекс в котором слово не помещается в свой контейнер
        breakIdx = getBreakIndex(text, rec);
        //если перенос есть до делаем дальнейшую проверку
        if (breakIdx > 0) {
            //к индексу надо прибавлять количество мягких переносов
            //чтобы получить реальное смещение
            //т.к они не отображаются как полноценный символ
            new_line = text.content.slice(breakIdx, text.content.length);
            //создаем новый текст
            newText = new PointText(new Point(text.bounds.x, text.bounds.y));
            newText.characterStyle = text.characterStyle;
            newText.content = new_line;

            if (lines.length==0) {
                lines.push(breakIdx);
            }
            else lines.push(lines[lines.length-1] + breakIdx);

            lines_obj.push(newText);

            //здесь надо брать следующую ячейку для переноса
            //если ее нет то выходим с 0 результатом (т.е успех)
            var next_hyp_field = rec;

            if (mode == 2) {

                //во втором режиме нужно переносить принимаю во внимание ширину связанной ячейки
                var has_links = rec.isLinked(rec.idx);

                if (has_links) {
                    var all_linked_fields = rec.getLinked(rec.idx);

                    if (all_linked_fields.length >= lines.length) {

                        next_hyp_field = recs[all_linked_fields[lines.length-1]];
                        //console.log("HYP-> next_fields -> " + next_hyp_field.idx);
                    }
                    else {
                        //console.log("HYP-> next_fields -> EXCEED!");
                        return -2;
                    }
                }
                else {
                    var current_line_idx = rec.linked_id[1];
                    var root = recs[rec.linked_id[0]];
                    var root_links = root.getLinked(root.idx);
                    next_hyp_field = recs[root_links[current_line_idx]];

                    if (!next_hyp_field) {
                        return -2;
                    }
                }
                //если связанные поля отсутствуют то берем

            }

            breakResult = autoBreakText(newText, next_hyp_field, lines, lines_obj, mode);
            //если больше переносов не найдено то возвращаем результаты
            //и останавливаем перебор

            if (breakResult == -1) {
                return 0;
            }
            return 0;
        }
        else return -1;
    }

    function hyphenTest() {
        var test_array = [],
            col_cnt = 0,
            row_cnt = 0;

        for (var r in recs) {
            if (recs[r].value) recs[r].value.remove();
            var val = "Тес-ти-ро-ва-ние ав-то-ма-ти-че-ско-го пе-ре-но-са слов #"+ row_cnt + ":#" + col_cnt;

            val = val.replace("-", hypsym);

            test_array.push(val);

            if (col_cnt >= columns_count-1) {
                row_cnt++;
                col_cnt = 0;
            }
            else col_cnt++;
        }
        return test_array;
    }

    function getHyphenator(f, text, linked) {
        //проба разместить текст в выбранной ячейке
        //индекс ячейки в глобальном пространстве
        //text - текст для размещения
        var field_text = '', analyzingText,
            lines, linesObj, breakResult, message;

        if (!linked) {
            field_text = f.value;
        }
        else {
            field_text = text;
        }

        analyzingText = new PointText(
            new Point(field_text._point.x,
                      field_text._point.y));

        analyzingText.characterStyle = field_text.characterStyle;
        analyzingText.content = field_text.content;

        lines = [];
        linesObj = [];
        breakResult = autoBreakText(analyzingText, f, lines, linesObj, 2);

        switch(breakResult) {
            case 0:
                message = 'Перенос был осуществлен!';
                break;
            case -1:
                message = 'Перенос не будет осуществлен!';
                break;
            case -2:
                message = 'Не хватает ячеек для расположения текста!';
                break;
            default:
                message = ''
        }

        var failed_codes = [-1, -2];

        var hypInfo = function() {
            var init;

            init = {
                field_id : f.idx,
                break_lines_idxs : lines,
                break_lines_text : linesObj,
                text : analyzingText,
                result_message : message,
                applied : failed_codes.indexOf(res)==-1,
                clearResults: function() {
                    for (var i = 0; i < this.break_lines_text.length; i++) {
                        this.break_lines_text[i].remove();
                    }
                    this.text.remove();
                }
            };
            return init;
        };

        return new hypInfo();
    }

    function singleHyphenation(cell) {
        //применение правил переноса к отдельной ячейке,
        //предполагаем что у ячейки есть связанная ячейка куда производиться
        //перенос лишнего текста. Функция работает рекурсивно, делает перенос
        //в соседнюю ячейку и потом в качестве рабочей принимает новую ячейку
        //@param - rec_id - индекс ячейки
        //если у данной ячейки нет связанной то не делаем перенос

        var hyphenator, analyzingText,
            f, cellId, rawText, userText, fieldText, nText, setText,
            linkedCells, valueForHyphenation,
            lines, linesObj, startIdx, endIdx,
            textLines, breakedText;

        f = cell;
        //ищем все связанные c этой ячейки
        linkedCells = f.getLinked();
        //если их нет то выходим
        if (linkedCells.length == 0) {return false;}
        //достаем текст поля из шаблона
        rawText = f.get('base_text');
        userText = f.get('user_text');//$(".hidden_data").slice(rec_id, rec_id + 1).html();
        //определяем значение для переноса
        valueForHyphenation = userText || rawText;
        //console.log("VALUE FOR HYPHENATION->" + value_for_hyphenation);
        //пока перенос происходит только по пробелам(заменяем пробелы символами мягкого переноса
        nText = valueForHyphenation.replace(/ /g, hypsym + " ");
        //обновляем текст в поле
        f.value.content = nText;

        fieldText = f.value;
        if (!fieldText) return;

        var cnt = 0;

        hyphenator = getHyphenator(f, fieldText, linkedCells);
        analyzingText = null;
        //если будет осуществлен
        if (hyphenator.applied) {

            analyzingText = hyphenator.text;
            lines = hyphenator.break_lines_idxs;
            linesObj = hyphenator.break_lines_text;
            startIdx = 0; endIdx = 0;
            textLines = [];
            breakedText = "\r\n";

            if (cnt == 0) {
                f.text_params.rows = [];
                f.text_params.rows_offset = [];
            }

            for (var r = 0; r < (lines.length + 1); r++) {

                if (r == lines.length) {
                    endIdx = analyzingText.content.length;
                } else {
                    linesObj[r].remove();
                    endIdx = lines[r];
                }

                textLines.push(analyzingText.content.slice(startIdx, endIdx));
                //FIXME - помоему ненужная переменная
                breakedText += analyzingText.content.slice(startIdx, endIdx);

                //ВСТАВЛЯЕМ НОВЫЙ ТЕКСТ (убираем пробелы)
                setText = analyzingText.content.slice(startIdx, endIdx);
                setText = setText.trim();

                if (setText.length && (setText[0] == hypsym)) {
                    setText = setText.slice(1, setText.length);
                }

                if (setText.length && (setText[0] == " ")) {
                    setText = setText.slice(1, setText.length);
                }

                var regexp = new RegExp(hypsym, 'g');
                var clear_text = setText.replace(regexp, '');

                f.text_params.rows.push(clear_text);
                f.text_params.rows_offset.push(-1);

                if (r <= lines.length) startIdx = lines[r];

            }

            analyzingText.content = setText;
            analyzingText.remove();
            //помечаем ячейку как подверженную переносу чтобы при переключении на редактирование
            //вернуть старое значение
            f.hyphenate = true;
        }
        else {
            hyphenator.clearResults();
            //если перенос не осуществлен то нужно проставить прежнее значение
            var user_text = f.text_params.rows[0];
            var val = f.get('value');
            val.content = user_text || rawText;
        }

        var textParams = f.get('text_params');
        for (var i = 0; i < (linkedCells.length + 1 - textParams.rows.length); i++) {
            textParams.rows.push('');
            textParams.rows_offset.push(-1);
        }

    }

    function applyHyphenation(start_from) {
        //применение к тексту правил переноса
        //проходим по всем прямоугольникам и высчитываем
        //помещается ли текст в заданные рамки, если нет то
        //переносим по правилам (если слово не подвержено правилу то
        //перенос осуществляется по пробелу)
        //также формируем зависимости между ячейками и раскидываем текст
        //по ним
        //start_from - индекс ячейки откуда мы начинаем считывать текст
        // console.log("APPLY HYPHENATION");
        var recs_values = $(".hidden_data");
        //не объявлена глобальная сортировка
        //console.log(hyphenation_on);
        if (hyphenation_on==undefined) return -1;

        var next_row_idx = 0;
        var next_page_row = 0;

        var rid = 0;
        var break_process = false;
        var text_info = [];
        var pre_vals = [];

        var offset = 0;
        var dop_offset = 0;

        if (start_from != undefined && start_from!=0) {
            offset = start_from;
            dop_offset = start_from - cell_rng_start;
        }
        else {
            offset = cell_rng_start;
            dop_offset = 0;
        }
        //TEST
        //cell_rng_end = 407;
        //получаем массив данных необходимых для переноса текста
        for (var t in recs) {
            //очищаем предыдущие данные переноса
            if (t<cell_rng_start || t>cell_rng_end) {
                pre_vals.push(recs_values.slice(t,t+1).html());
                continue;
            }

            recs[t].linked_id = [-1, 0];
            recs[t].text_params.rows = [];
            recs[t].text_params.rows_offset = [];

            var new_idx = dop_offset + parseInt(t);

            // console.log(recs_values.slice(new_idx,new_idx+1).html());
            pre_vals.push(recs_values.slice(new_idx,new_idx+1).html());

            // if (t < 100)
            // 	console.log("GET DATA FOR #" + t + ", FOR->" + new_idx + ", TxT" +
            // 		recs_values.slice(new_idx,new_idx+1).html());
        }
        //заполняем прямоугольники данными
        loadData(pre_vals);

        //удаляем весь текст из полей и копируем его в промежуточный массив
        for (var t in recs) {
            text_info.push(recs[t].value);
            // console.log("TEXT VALUE=" + recs[t].value.content);

            //если поле лежит вне диапазона тех полей которые требуют переноса
            //то не очищаем весь текст
            if (t < cell_rng_start || t > cell_rng_end) continue;

            if (recs[t].value) recs[t].value.remove();
        }
        //какое следующее поле использовать для размещения текста
        // console.log("HYP_SETUP->COL_CNT=" + columns_count + ", CELL START=" + cell_rng_start
        // 	+ ", CELL END=" + cell_rng_end);

        var table_row_cnt = 0;
        var cnt = 0;
        var next_data_idx = 0;
        var counter = 0;
        //цикл прерывается когда переносимые поля не умещаются
        while (!break_process) {
            //индекс новой строки куда мы складываем новые данные
            //индекс из таблицы данных откуда берем текст
            next_data_idx = offset + cnt * columns_count;

            // console.log("!!!!!!----NEXT_DATA_ROW -> " +
            // 	"\r\nIDX   -> " + next_data_idx + ",START=" + start_from + ",OFFSET=" + offset + ", PRE_MAX_ROWS = " + next_row_idx
            // );

            var max_lines_col = 0;
            var pre_lines_col = 1;

            for (var col = 0; col < columns_count; col++) {
                //индекс нового прямоугольника от которого отсчитывается новая строка
                rid = (cnt * columns_count + cell_rng_start) + next_row_idx*columns_count + col;

                // console.log("ROW="+ next_row_idx + ", COL=" + col + ", ID=" + rid +
                // 	", DATA->" + text_info[cell_rng_start+cnt*columns_count+col].content//[next_data_idx + col].content
                // );

                //если cell_rng_start = 0 и cell_rng_end = 0
                //то перенос не применятся
                if (rid >= cell_rng_start && rid <= cell_rng_end && cell_rng_start!=0 && cell_rng_end!=0) {

                    //START singleHyphenation (КОД С ЭТОЙ МЕТКИ ВЫДЕЛЕН В ОТДЕЛЬНУЮ ФУНКЦИЮ)
                    var lines = [];
                    var lines_obj = [];
                    var ftext = text_info[cell_rng_start+cnt*columns_count+col];
                    var res = -1;
                    //заполняем массив строк,он передается по ссылке
                    res = autoBreakText(ftext,recs[rid],lines,lines_obj);

                    // console.log("BREAKS -> " + res + ", BREAK LINES->" + lines.length);

                    var text_lines = [];
                    var breaked_text = '\r\n';
                    var start_idx = 0;
                    var end_idx = 0;
                    //функция autoBreakText возвращает список индексов где надо сделать перенос
                    //переносим слова и распределяем в ячейки
                    for (var r=0; r < (lines.length+1); r++) {

                        if (r == lines.length) end_idx = ftext.content.length;
                        else {
                            lines_obj[r].remove();
                            end_idx = lines[r];
                        }

                        text_lines.push(ftext.content.slice(start_idx,end_idx));
                        breaked_text += ftext.content.slice(start_idx,end_idx) + '\r\n';

                        //console.log("	SET LINKED LINES #"+ recs[rid].text_params.rows.length + "->" + rid + ", DATA=" + ftext.content.slice(start_idx,end_idx));

                        //ВСТАВЛЯЕМ НОВЫЙ ТЕКСТ (убираем пробелы)
                        var set_text = ftext.content.slice(start_idx,end_idx);
                        set_text = set_text.trim();//replace(/^\s+|\s+&/g,'');

                        if (set_text.length && (set_text[0]==String.fromCharCode(0x00ad)))
                            set_text = set_text.slice(1,set_text.length);

                        if (set_text.length && (set_text[0]==" "))
                            set_text = set_text.slice(1,set_text.length);

                        recs[rid].text_params.rows.push(set_text);
                        recs[rid].text_params.rows_offset.push(-1);

                        if (r <= lines.length) start_idx = lines[r];
                    }
                    //END singleHyphenation (КОД С ЭТОЙ МЕТКИ ВЫДЕЛЕН В ОТДЕЛЬНУЮ ФУНКЦИЮ)

                    // console.log("	BREAKED TEXT->" + breaked_text);
                    //определяем максимальное количество строк в столбце
                    //для того чтобы правильно задать смещение для размещения следующих данных
                    if (!lines.length) lines = [0];
                    if (lines.length > max_lines_col) max_lines_col = lines.length;
                    //если последняя колонка то считаем максимальное
                    //количество строк переноса из всех колонок в строке
                    if (col == (columns_count - 1)) {
                        //console.log("SET MAX LINES->" + max_lines_col);
                        next_row_idx += max_lines_col;
                    }

                    pre_lines_col = lines.length;
                }
                else {
                    //console.log("maximum end edge detect");
                    break_process = true;
                    // console.log("BREAK CYCLE0");
                    break;
                }
            }

            //если достигнут конец таблицы
            if (break_process) {
                // console.log("CNT->" + cnt);
                //cnt++;
                break;
            }
            else {cnt++;}//console.log("INCRIMENT CNT->" + (cnt));}

            //console.log("\r\n~~~~~~RESULT~~~~~~\r\n");

            if ((parent_id + max_lines_col*columns_count)
                > cell_rng_end) {
                break_process = true;
                recs[parent_id].text_params.rows.push(' ');
                //break;
            }
            //расставляем признаки зависимости

            for (var n = 0; n < columns_count; n++) {
                var parent_id = rid - columns_count + n + 1;

                if (parent_id > cell_rng_end) {
                    //if (!break_process) next_page_row+=1;
                    break_process = true;
                    break;
                }

                var parent = recs[parent_id];
                var childs = [];

                if (max_lines_col!=0) {
                    recs[parent_id].text_params.rows.push(' ');
                }

                for (var r = 0; r < max_lines_col; r++) {

                    if (recs[parent_id].text_params.rows.length==0) {
                        recs[parent_id].text_params.rows.push(' ');
                    }
                    //добавляем недостающие ячейки
                    if (recs[parent_id].text_params.rows.length < (max_lines_col+1)) {
                        recs[parent_id].text_params.rows.push(' ');
                    }

                    var child_id = parent_id + (r+1)*columns_count;

                    if (((parent_id + max_lines_col*columns_count)
                        > cell_rng_end ) && child_id <= cell_rng_end) {
                        //if (!break_process) next_page_row+=1;
                        break_process = true;
                        // console.log("BREAK_CYCLE1");
                        recs[parent_id].text_params.rows = [' '];
                        break;
                    }

                    if (child_id > cell_rng_end) {
                        //if (!break_process) next_page_row+=1;
                        break_process = true;
                        // console.log("BREAK_CYCLE2");
                        recs[parent_id].text_params.rows = [' '];
                        break;
                    }

                    var child = recs[child_id];

                    child.text_params.rows = [];
                    child.text_params.rows_offset = [];

                    child.linked_id = [parent_id, r+1];

                    // console.log("PARENT_ID[" + recs[parent_id].text_params.rows.length + "]->" + parent_id + ", CHILD->" + (parent_id + (r+1)*columns_count) + ", LINKED->" + child.linked_id + ", DATA->" + recs[parent_id].text_params.rows[child.linked_id[1]]);
                }
            }
            //
            if (!break_process) {
                // console.log("SET ROW ->"
                // 	+ next_data_idx + ", "
                // 	+ (next_data_idx + columns_count) + ", MAXL="
                // 	+ (max_lines_col)
                // );
                next_page_row = next_data_idx + max_lines_col - 1 + offset;
                next_page_row = next_data_idx + columns_count;// + cnt*columns_count;
            }

            //считаем количество пройденных строк
            table_row_cnt=cnt;
        }
        //ЗАПОЛНЯЕМ ДАННЫЕ СНОВА ПОСЛЕ ПЕРЕНОСА
        loadData(['hid_hyp']);
        //changePrintStyle(['print_mode']);
        return next_page_row;
    }

    return {
        autoBreakText: autoBreakText,
        getBreakIndex: getBreakIndex,
        getHyphenator: getHyphenator,
        applyHyphenation: applyHyphenation

    }
});