/**
 * Модель редактируемой ячейки в редакторе бланков
 */

define('models/cell_model', ['backbone', 'paper', 'helpers'],

    function(Backbone, Paper, helpers) {

    /**
     * Поле редактора доступное для редактирования (ввод текста, перемещение)
     */
    var CellModel = Backbone.Model.extend({
        defaults: {
            showTooltip     : true,
            manager         : null,
            idx             : 0,
            center          : null,
            width           : 20,
            height          : 20,
            fixed           : false,
            unselect_color  : '#000000',
            group_          : null,
            //текст который вставляется в поле при печати
            value           : null,
            //объемлющий прямоугольник
            boundedRectangle: null,
            id_text         : null,
            //связанное поле из которого берется текст,
            //первый индекс - номер поля,
            //второй индекс - номер строки
            linked_id       : null,
            print           : true,
            descr           : "",
            hyphenate       : false, //признак переноса слов
            base_text       : "",    //текст при первоначальной загрузке
            user_text       : "",     //текст введенный пользователем
            lines           : [],
            //параметры текста, такие как тип шрифта и его размер
            text_params     : null,
            offsetType      : 'absolute',
            printOffsetLeft : 0,
            printOffsetTop  : 0
        },
        initialize: function() {
            this.idx = this.cid;
            this.set('linked_id', [-1, 0]);
            this.set('text_params', _.extend({}, {
                //строки текста на которые разбита общая строка
                rows: [],
                //отступ от края поля
                rows_offset: [],
                font_type: "",
                font_size: ""
            }));
        },
        getWidth: function() {          
            return helpers.to_hUnit(this.get('width').toFixed(2));
        },
        getHeight: function() {
            return helpers.to_hUnit(this.get('height').toFixed(2));
        },
        getCenterX: function() {
            var center = this.get('center'),
                width = this.get('width');

            return center ? helpers.to_hUnit(center.x.toFixed(2)-width/2) : 10;
        },
        getCenterY: function() {
            var center = this.get('center'),
                height = this.get('height');

            return center ? helpers.to_hUnit(center.y.toFixed(2)-height/2) : 10;
        },
        removeSafe: function() { 
            var group_ = this.get('group_');
            if (group_) {
                group_.remove();
            };
        },
        updateCenter: function(self_id, new_center_point) {
            this.center = new_center_point;
        },
        updateLeft: function(self_id, new_left_point) {
            this.center = new Point(
                new_left_point + this.width/2,
                this.center.y);
        },
        updateWidth: function(self_id, new_width) {
            this.width = new_width;
        },
        updateTop: function(self_id, new_top_point) {
            this.center = new Point(
                this.center.x,
                new_top_point + this.height/2);
        },
        updateHeight: function(self_id, new_height) {
            this.height = new_height;
        },
        getLinkId: function() {
            var link = this.get('linked_id');
            return link[0];
        },
        leftOffset: function() {
            var center = this.get('center'),
                width = this.get('width');

            return center.x - width / 2;
        },
        topOffset: function() {
            var center = this.get('center'),
                height = this.get('height');

            return center.y - height / 2;
        },
        getFontSize: function() {
            var text_params = this.get('text_params');
            return text_params.font_size || 12;
        },
        getFontType: function() {
            var text_params = this.get('text_params');
            return text_params.font_type || 'Arial';
        },
        getLinked: function(self_id) {
            //метод возвращает связанные с данной ячейкой другие ячейки
            var lst = [], mgr = this.manager;
            for (var r in mgr) {
                if (mgr[r].linked_id[0] == self_id) {
                    lst.push(parseInt(r));
                }
            }
            return lst;
        },
        isLinked: function(self_id) {
            //хелпер показывающий является ли ячейка связанной
            return this.linked_id[0] == -1;
        },
        isNotPrinted: function(no_formatting) {
            return !this.isPrinted;
        },
        isPrinted: function(no_formatting) {
            var checkbox_value = "checked='checked'",
                val = this.get('print'), ret_val;

            if (val) {
                return no_formatting ? val : '';
            }
            ret_val = no_formatting ? false : checkbox_value;

            return ret_val;
        },
        copy: function(source, target) {
            //копирование основных параметров
            target.set('center', source.getCenter());
            target.set('width', source.getWidth());
            target.set('height', source.getHeight());
            target.set('lines', source.get('lines'));
            target.set('group_', source.get('group_'));
            target.set('value', source.get('value'));
            target.set('boundedRectangle', source.get('boundedRectangle'));
        },
        getChildrens: function() {
            var childs = this.get('group_').children;
            return childs;
        }
    });

    return CellModel;
});