

define('models/layer_model', ['jquery', 'backbone'], function($, Backbone) {

    var onLoadDataComplete = function(data) {
        var that = onLoadDataComplete.this_;

        if (!data._cached) {

            if (data['success'] === false) {
                alert("При загрузке данных возникли проблемы!");
                return ;
            }
            //data - представляем из себя 2 списка один
            //1ый - список значений полей
            //2ой - список описания полей
            //получаем данные из полей
            //если data передается в виде единого списка то предполагаем что описания полей отсутствует
            //а есть только значения
            var values = {}, descriptions = {};

            if (data.length == 1) {
                values = data;
            }
            else if (data.length == 2) {
                values = data[0];
                descriptions = data[1];
            }
            var i = 0, text_params;

            that.objectsCollection.each(function(model) {
                text_params = model.get('text_params');
                model.set('base_text', values[i]);
                model.set('desc', descriptions[i]);
                i++;
            });

            that._cached = true;
            that.trigger('data_loaded', that);
        }

    };

    var DefaultDataLoader = function(loadDataUrl, loadDataParams) {

        var ajaxParams = {
            url: loadDataUrl,
            dataType: 'json',
            type: 'POST',
            data : loadDataParams
        };

        return {
            load: function(callbackOk, callbackFail) {
                return $.when(
                $.ajax(ajaxParams))
                .then(callbackOk, callbackFail);
            }
        }
    };

    var model = Backbone.Model.extend({

        defaults: {
            name: 'dummy_layer',
            objectsCollection: null,
            _cached: false,
            checkLoader: false,
            dataLoader: null,
            loadDataUrl: '',
            loadDataParams: ''
        },
        getObjects: function() {
            /**
             * хелпер для возможности извлечь в виде списка все объекты с
             * заданного слоя
             */
        },
        
        onLoadDataFailed: function() {
            alert("Ошибка при загрузке данных для шаблона!");
        },

        loadDataToObjects: function() {
            /**
             * удаленная загрузка данных в объекты слоя
             */
            var promise;

            if (!this.dataLoader && this.checkLoader) {
                throw new Error("You must define a dataLoader module!")
            }
            //если не задан загрузчик то используем загрузчик по умолчанию
            if (!this.dataLoader) {
                this.dataLoader = new DefaultDataLoader(
                    this.get('loadDataUrl'),
                    this.get('loadDataParams')
                );
            }
            //загрузчик должен возвращать Promise объект
            //если данные раннее были загружены то Promise сразу резолвим
            onLoadDataComplete.this_ = this;

            if (!this._cached) {
                promise = this.dataLoader.load(
                    onLoadDataComplete,
                    this.onLoadDataFailed
                )
            } else {
                promise = $.Deferred().resolve([{'_cached': true}]);
            }

            return {
                promise: promise.promise(),
            }
        }
    });

    return model;
});
