/**
 * Представление для панели инструментов
 * Список координат
 */
define('views/cell_view', ['jquery', 'helpers', 'backbone', 'handlebars', 'models/cell_model'],
    function($, helpers, Backbone, _Handlebars, CellModel) {

        var template = [
            "<td class='col_check'>{{#unless fixed}}<input type='checkbox'/>{{/unless}}</td>",
            "<td class='id_field'>{{idx}}</td>",
            "<td class='cell_center_x'><input type='text' value='{{centerX}}'></td>",
            "<td class='cell_center_y'><input type='text' value='{{centerY}}'></td>",
            "<td class='cell_width'><input type='text' value='{{width}}'></td>",
            "<td class='cell_height'><input type='text' value='{{height}}'></td>",
            "<td class='cell_islink'>{{getLinked}}</td>",
            "<td class='cell_print_ex'><input type='checkbox' value='{{isPrinted}}'/></td>",
            "<td class='description'>{{desc}}</td>"
        ].join("");

        var view = Backbone.View.extend({
            
            tagName: 'tr',
            className: 'cell_info',
            modelType: CellModel,
            template: template,

            events: {
                'keypress .cell_center_x input'    : 'updateCenterX',
                'keypress .cell_center_y input '   : 'updateCenterY',
                'keypress .cell_width input '      : 'updateRWidth',
                'keypress .cell_height input '     : 'updateRHeight',
                'change .cell_print_ex input '     : 'printExclude'
            },
            initialize: function(config) {
                this.parentView = config['parentView'];
            },
            render: function() {

                this.model.on('change', this.updateCellParams, this);
                this.$el.attr("id", "cinfo" + this.model.idx);
                var template = Handlebars.compile(this.template),
                    model_dump = this.model.toJSON();

                console.dir(model_dump);
                model_dump = _.extend(model_dump, {
                    'width': this.model.getWidth(),
                    'height': this.model.getHeight(),
                    'centerX': this.model.getCenterX(),
                    'centerY': this.model.getCenterY(),
                    'getLinked' : this.model.getLinked(),
                    'isPrinted': this.model.isPrinted()
                });

                this.$el.html(template(model_dump));

                return this;
            },
            updateCenterX: function(event) {
                if ((event.keyCode || event.which) == 13) {
                    var nval = helpers.to_pUnit(parseFloat(this.$('.cell_center_x input').val())),
                        center = this.model.get('center'),
                        width = this.model.get('width'),
                        old_val;

                    old_val = center.x;
                    center.x = nval + width/2;
                    this.model.set('center', center, {silent: true});
                    //выстреливаем события чтобы компоненты системы
                    //могли отреагировать на изменение
                    this.parentView.trigger('change:center_x', this, this.model, nval, old_val);
                }
            },
            updateCenterY: function(event) {
                if ((event.keyCode || event.which) == 13) {
                    var val = helpers.to_pUnit(parseFloat(this.$('.cell_center_y input').val())),
                        center = this.model.get('center'),
                        height = this.model.get('height'),
                        old_val;

                    old_val = center.y;
                    center.y = val + height/2;
                    this.model.set('center', center, {silent: true});
                    this.parentView.trigger('change:center_y', this, this.model, val, old_val);
                }
            },
            updateWidth: function(event) {
                if ((event.keyCode || event.which) == 13) {
                    var val = helpers.to_pUnit(parseFloat(this.$('.cell_width input').val())), old_val;
                    old_val = this.model.get('width');
                    this.model.set('width', val, {silent: true});
                    this.parentView.trigger('change:width', this, this.model, val, old_val);
                }
            },
            updateHeight: function(event) {
                if ((event.keyCode || event.which) == 13) {
                    var val = helpers.to_pUnit(parseFloat(this.$('.cell_height input').val())), old_val;
                    old_val = this.model.get('height');
                    this.model.set('height', val, {silent: true});
                    this.parentView.trigger('change:height', this, this.model, val, old_val);
                }
            },
            updateCellParams: function(model, attrs) {
                //отслеживание изменений состояния модели
                //и внесение изменений в представление
                var width, height, center;

                width = model.get("width");
                height = model.get("height");
                center = model.get('center');

                if (width) {
                    this.$(".cell_width input").val(
                        helpers.to_hUnit(width).toString());
                } 

                if (height) {
                    this.$(".cell_height input").val(
                        helpers.to_hUnit(height).toString());
                } 

                if (model.get("center")) {
                    this.$(".cell_center_x input").val(
                        helpers.to_hUnit(center.x - width / 2).toString());
                    this.$(".cell_center_y input").val(
                        helpers.to_hUnit(center.y - height/ 2).toString());
                }
            },
            printExclude: function(event) {
                /* проставляем признак все полям которые отключены для печати */
                this.model.set('print', $(event.currentTarget).attr("checked") == "checked");
            }
        }, {

        });

        return view
    }
);