
define('views/cellinfo_view', ['jquery', 'backbone', 'views/cell_view', 'views/tool_view'],
	function($, Backbone, CellView, ToolView) {

		var template = [
			"<h3>Положение ячеек</h3>",
			"<div>",
				"<table border=2>",
	            "<tr>",
	            	"<td class='header_check'><input type='checkbox'/></td>",
	                "<td>№поля</td>",
	                "<td>Слева</td>",
	                "<td>Сверху</td>",
	                "<td>Ширина</td>",
	                "<td>Высота</td>",
	                "<td>Связь</td>",
	                "<td>Искл.</td>",
	                "<td class=\"description\">Описание</td>",
	            "</tr>",
	            "</table>",
            "</div>"
        ].join("\r\n");

		var CellInfoView = ToolView.extend({

			template: template,
			id: 'coord_info',
			tagName: 'div',
			className: 'cellsTable',
			events: {
                'click #extended_tools'     : 'showView',
				'click .add_cell_btn'		: 'addCell',
				'click .remove_cell_btn'	: 'removeAll',
				'change .header_check'		: 'selectAll'
			},
			initialize: function(config) {
				this.views = [];
				this.collection = config.collection;
				this.collection.view_ = this;
			},

			afterRender: function() {
				this.selectHeaderCheckbox = this.$('.header_check input');
				this.addButton = this.$el.prepend("<button class='add_cell_btn'>Добавить</button>");
				this.removeButton = this.$el.prepend("<button class='remove_cell_btn'>Удалить</button>");
                this.$el.addClass('xt_view');
                this.$el.css('display', 'none');
			},

			render: function() {
				this.$el.html(this.template);
				this.$el.css("padding", '1em');
				return this;
			},

			getSelectedIds: function() {

				var rows = this.$('tr'),
					selected, idx, selected_ids = [],
					cnt = 0;

				rows.each(function(row) {

					selected = $(this).find('td input').eq(0);
					idx = $(this).find('td id_field').eq(0);

					if (selected.attr('checked') == "checked") {
						selected_ids.push(cnt-1);
					}

					cnt += 1;
				});

				return selected_ids;
			},

			addCell: function(event, model_config) {
				var model, cellView, rendered_view;

				model = new this.collection.modelType(model_config || {
					idx: this.collection.length + 1
				});
				cellView = new CellView({model: model, parentView: this});
				rendered_view = cellView.render().el;

				this.collection.add(model);
				$(rendered_view).addClass("cellview_" + cellView.model.cid);
				this.$('table').append(rendered_view);

				//для обновления вьюхи перепишем один из аттрибутов чтобы сработал change
				model.set({idx: model.get('idx')}, {changed: true});
				this.views.push(cellView);
			},
			selectCell: function(cellview) {
				//выбор ячейки
				var checkbox = cellview.find('td').eq(0).find('input[type="checkbox"]');

				if (this.selectHeaderCheckbox.attr('checked') == "checked") {
					checkbox.attr("checked", 'checked');
				} else {
					checkbox.removeAttr("checked");
				}
			},
			removeCell: function(cellview) {
				//удаление ячейки

				var checkbox = cellview.find('td').eq(0).find('input[type="checkbox"]');
				var viewClasses = cellview.attr("class").split("cellview_");
				var cidClass = viewClasses.slice(viewClasses.length-1, viewClasses.length)[0];

				if (checkbox.attr("checked") == "checked") {
					var view;
					for (var v = 0; v < this.views.length; v++) {
						view = this.views[v];
						if (view.model.cid == cidClass) {
							view.model.removeSafe();
							this.collection.remove(view.model.cid);
							view.remove();
						}
					}
				}
			},
			applyToCells: function(func) {
				var rows = this.$('tr').slice(1, this.$('tr').length), that = this;

				rows.each(function() {
					func.call(that, $(this));
				});
			},
			selectAll: function() {
				this.applyToCells.apply(this, [this.selectCell]);
			},
			removeAll: function() {
				this.applyToCells.apply(this, [this.removeCell]);
			},
            setFontParams: function(font_type, font_size) {

                this.collection.each(function(cell) {
                    var text_params = cell.get('text_params');
                    text_params.font_type = font_type;
                    text_params.font_size = font_size;

                    cell.set('text_params', text_params);
                })
            },
            getAssociatedView: function(model) {
            	//получаем вьюху для изменившейся модели
            	var cid = model.cid;
            	var search_view = ".cellview_"+cid;

            	return this.$(search_view);
            },
            updateCellDescription: function(model, desc) {
            	var view_ = this.getAssociatedView(model);
            	view_.find('.description').html(desc);
            }
		});

		return CellInfoView;
	}
);