define('views/editor_view', ['jquery', 'backbone', 'helpers', 'views/tool_view'],
	function($, Backbone, helpers, ToolView) {

		function _initEditor() {

	        //настраиваем размер канваса в зависимости от реального разрешения экрана
	        this.dpi = helpers.getDpi();
	        //количество пискелей в одном миллиметре
	        var layerConfig = this.model.get('layer'),
				pixel_size = this.dpi / 25.4,
	            canvas = this.canvas,
	            canvas_container = this.container,
	            sub_picture_size = this.model.get('workspace_size'),
	            blank_size = layerConfig.canvas_default_size,
	            blank_width = parseFloat(blank_size.split(',')[0]),
	            blank_height = parseFloat(blank_size.split(',')[1]),
	            blank_pixel_width = blank_width * pixel_size,
	            blank_pixel_height = blank_height * pixel_size,
	            sub_picture_width = parseFloat(sub_picture_size.split(',')[0]),
	            sub_picture_height = parseFloat(sub_picture_size.split(',')[1]),
	            sub_pixel_width = sub_picture_width * pixel_size,
	            sub_pixel_height = sub_picture_height * pixel_size;

	        //задание размера для канваса
	        //используем настройки канваса по умолчанию
	        canvas.attr("width", Math.floor(
	            sub_pixel_width.toFixed(2)).toString() + "px");
	        canvas.attr("height", Math.floor(
	            sub_pixel_height.toFixed(2)).toString() + "px");

	        var sub_picture_lft = (blank_pixel_width - sub_pixel_width)/2;

	        //позиционируем подложку так чтобы она располагалась по середине

	        canvas_container.css("width",
	            Math.floor(blank_pixel_width.toFixed(2)).toString() + "px");
	        canvas_container.css("height",
	            Math.floor(blank_pixel_height.toFixed(2)).toString() + "px");

	        if (sub_picture_lft > 0) {

	            var left_val = Math.floor(sub_picture_lft.toFixed(2)).toString() + "px";

	            canvas.css("left", 0);
	            canvas_container.css('left', left_val);
	        }
	        //получим полный путь до бекграунда
	        var full_path_to_background = "url(\"+" + mediaUrl + this.model.get('background_img_url') + "\")";
	        //если путь при загрузке пустой раннее сохраненный бекграунд
	        canvas.css("background-image", full_path_to_background);
	        $("info").css("width", "0px");

		}

		var template = [
				"<canvas id='paperCanvas' class='canvas' width='100' height='100'>",
				"</canvas>",
				"<div id='print2side'>",
					"<input id='print2side_chk' type='checkbox'/>",
					"<p style='display: block; float:left; margin: 0 10px;'>Печать второй стороны</p>",
				"</div>"
        	].join("\r\n");

		var EditorView = ToolView.extend({
			id: 'blank_container', 
			template: template,
			events: {
				'click #print_control'			: 'print',
				'change #print2side'			: 'set2SideView'
			},
			afterRender: function() {
				this.container = $("#blank_container");
				this.canvas = $("#paperCanvas");

				this.showTwoSideCheckbox = $("#print2side");
				this.setOffsetCheckbox = $("#set_page_glparams");
				this.toggleEditModeCheckbox = $("#set_editor_mode");
				this.printButton = $("#print_control");

				this.initEditor();
			},
			render: function() {
				this.$el.html(this.template);
				return this;
			},
			initEditor: _initEditor,
	        isEditMode : function() {
	            return this.model.get('editMode');
	        },
	        print : function() {
	            //обработка нажатия на кнопку печать
	            var edit_mode = this.isEditMode();
	            if (!edit_mode) {
	                window.print();
	            }
	            else {
	                //TODO - заменить на вьюху ошибки
	                alert("Для печати бланка отключите режим \"Редактирование\"!" +
	                    "\r\nДля этого включите флажок \"Печать/редактирование\"");
	            }
	        },
            setOffsetParams: function(offset_mode) {
                //применение глобальных настроек смещения полей
                //сами значения параметров отступов и масштабов хранятся в модели EditorModel
                //за включение режима отвечает сторонее представление (оно вырабатывает событие
                //через шину событий pubSub)

                var ths = this.setOffsetModeCheckbox,
                    left_offset = 0, top_offset = 0,
                    model = this.model;

                if (offset_mode) {

                    left_offset = model.get('offset_x');
                    top_offset = model.get('offset_y');

                    var image_offset_x = 0,
                        image_offset_y = 0;

                    this.canvas.css("backgroundRepeat", "no-repeat");
                    this.container.css("backgroundRepeat", "no-repeat");

                    var background_pos = helpers.to_pUnit((image_offset_x + left_offset).toString())
                            + "px " + helpers.to_pUnit((image_offset_y + top_offset).toString()) + "px";

                    this.canvas.css("backgroundPosition", background_pos);
                    this.container.css("backgroundPosition", background_pos);

                    model.set('offsetMode', true);

                }
                else {
                    //сбрасываем режим Включение отступов и масштабов
                    this.canvas.css("backgroundPosition", "0px 0px");
                    this.container.css("backgroundPosition", "0px 0px");

                    model.set('offsetMode', false);
                }

            },
		    set2SideView: function() {
		        //показываем и скрываем 2-ю сторону бланка
		        var canvas = this.canvas,
		        	current_render_mode = this.model.get('render_mode'),
		        	canvas_container = this.container,
		            chk = $("#print2side_chk"),
                    twoSide, top_oset,
		            cvs_height = parseFloat(canvas.css("height").replace(/px/, ""));

                twoSide = chk.attr("checked") == "checked";
		        if (twoSide) {
		            top_oset = -1 * cvs_height/2;
		            canvas.css("marginTop", top_oset.toString() + "px");

		            if (current_render_mode == "DIVS") {
		                canvas_container.css("marginTop", top_oset.toString() + "px");
		            }
		        }
		        else {
                    top_oset = 0;
		            canvas.css("marginTop",     "0px");
		            if (current_render_mode == "DIVS") {
		                canvas_container.css("marginTop", "0px");
		            }
		        }

                this.model.set('twoSideMode', twoSide);
                this.model.set('twoSideOffset', top_oset);
		    },
        	getActions: function() {
	            //адреса для Ajax запросов на сервер
	            return {
	                save_template: this.model.get('save_template_url'),
	                load_template: this.model.get('load_template_url'),
	                upload_blank:  this.model.get('upload_blank'),
	                get_params_url: this.model.get('get_params_url'),
	                provider_url: this.model.get('provider_url')
	            }
	        }

		});

		return EditorView;
	}
);