

define('views/layer_manager_view', ['jquery', 'backbone',
        'collections/layer_collection',
        'models/text_layer',
        'models/cell_layer',
        'views/tool_view'
    ],
    function($, Backbone, LayersCollection,
             TextLayerModel, CellLayerModel,
             ToolView
        ) {
        /**
         * представление для работы со слоями редактора
         * к основным слоям относят текстовый слой который
         * выводится при печати и слой ячеек (слой редактирования)
         */

        var layerCollection = new LayersCollection([
            new TextLayerModel({
                "name": 'text_layer'
            }),
            new CellLayerModel({
                "name": 'cell_layer'
            })
        ]);

        var template = [
            '<h3>Переключение режима</h3>',
            '<div>',
                '<div class="instrument_line">',
                    '<div style="width: 40%; float:left;">',
                        '<div><button class="print_button">Печать</button></div>',
                        '<div>',
                            '<p><b>Печать/Редактирование</b></p>',
                        '</div>',
                        '<div>',
                            '<input id="set_editor_mode" text="Для печати" type="checkbox" checked="checked"/>',
                        '</div>',
                    '</div>',
                    "<div style='width: 40%' id='extended_tools'><label style='font-weight: bold;'>Расширенные настройки<input type='checkbox'/></label></div>",
                    '<div style="width: 20%; float:left;">',
                        '<div class="hidden">',
                            '<p><b>Вкл.сжатие/отступы</b></p>',
                        '</div>',
                        '<div class="hidden" style="margin-right: 2em;">',
                            '<input id="set_page_glparams" text="Глоб.настройки" type="checkbox">',
                        '</div>',
                        '<div class="hidden">',
                            '<label for="page_selector">Выбор страницы</label>',
                            '<input id="page_selector" size=3/>',
                        '</div>',
                    '</div>',
                '</div>',
            '</div>'
        ].join("\r\n");

        var LayerManagerView = ToolView.extend({
            template: template,
            tagName: 'div',

            events: {
                'click .print_button'           : 'print',
                'change #extended_tools input'  : 'showExtended',
                'change #set_editor_mode'       : 'onChangeModeEventHandler',
                'change #set_page_glparams'     : 'changeOffsetMode'
            },

            initialize: function(config) {

                this.drawingDelegate = config['drawer'];
                this.editor = config['editor'];
                this.collection = layerCollection;
                //TODO - по умолчанию слой должен быть текстовый
                this.collection.activeLayer = this.collection.at(1);

                this.collection.each(function(layer) {
                    layer.on('activate', layer.onActivate, [this, layer, this.collection]);
                    layer.on('deactivate', layer.onDeactivate, [this, layer, this.collection]);
                    //всем слоям нужно обеспечить доступ к коллекции объектов
                    layer.drawingDelegate = config['drawer'];
                    layer.objectsCollection = config['objectsCollection'];
                    layer.set('loadDataUrl', config['loadDataUrl']);
                    layer.set('loadDataParams', config['loadDataParams']);
                }, this);
                //подписка на изменение режима редактора в контексте коллекции ячеек
                this.editor.model.on('change:editMode', this.changePrintMode, this);

            },
            afterRender: function() {
                this.setPrintModeCheckbox = this.$('#set_editor_mode');
                this.setOffsetModeCheckbox = this.$('#set_page_glparams');
                this.setPrintModeCheckbox.attr('checked', 'checked');
                this.extendedShow = this.$("#extended_tools input");
            },
            render: function() {
                this.$el.html(template);
                this.afterRender();
                this.$el.css('padding', '1em');
                return this;
            },
            print: function() {

                var edit_mode = this.editor.isEditMode();
                if (!edit_mode) {
                    window.print();
                }
                else {
                    //TODO - заменить на вьюху ошибки
                    alert("Для печати бланка отключите режим \"Редактирование\"!" +
                        "\r\nДля этого включите флажок \"Печать/редактирование\"");
                }

            },
            showExtended: function() {
                var extended_tools = $("body .xt_view");

                if (this.extendedShow.attr('checked') != "checked") {
                    extended_tools.css('display', 'none');
                } else {
                    extended_tools.css('display', 'block');
                }
            },
            onChangeModeEventHandler: function() {
                var is_print = this.setPrintModeCheckbox.attr('checked') == "checked";
                this.editor.model.set('editMode', !is_print);
            },
            changePrintMode: function(editorModel, editMode) {
                //данный метод переключает состояние ячеек в разных режимах редактора
                var layer, that = this;
                //сбрасываем состояние активного слоя
                this.collection.activeLayer.set('active', false);

                if (!editMode) {
                    layer = this.collection.at(0);
                } else {
                    layer = this.collection.at(1);
                }

                this.collection.activeLayer = layer;

                layer.trigger('activate', this, layer);

                //посылаем сигнал на деактивацию модели
                this.collection.each(function(layer) {
                    if (layer !== this.activeLayer) {
                        layer.trigger('deactivate', that, layer);
                    }
                }, this.collection);

            },
            changeOffsetMode: function() {

                //Установка отступов для подгонки под настройки принтера
                //обработка нажатия на чекбокс "Установить отступы"
                var el = this.setOffsetModeCheckbox,
                    is_edit_mode = this.editor.isEditMode();

                if (!is_edit_mode) {
                    alert("Установка отступов невозможна в режиме печати!\r\n" +
                        "Включите режим \"Редактирование\"");
                    //инвертируем значение
                    if (el.attr("checked") != "checked") {
                        el.removeAttr("checked");
                    } else {
                        el.attr("checked", "checked");
                    }

                    return false;
                }

                this.pubSub.trigger('offset_mode:change', el.attr("checked") == "checked");
            }
        });

        return LayerManagerView;
    }
);
