define('views/offset_setup_view', ['jquery', 'backbone', 'views/tool_view'],

	function($, Backbone, ToolView) {
		/**
		 * Представление для настройки отступов и сжатий
		 */
		var template = [
			'<div style="height: 120px;">',
			'<h3>Настройка отступов и масштабов</h3>',
			'<div>',
				'<div class="instrument_line" style="width: 70%; float:left; height: 100%;">',
					'<div class="tbl_opt_label" ><p class="inline">СжатиеX</p></div>',
					'<div class="tbl_opt" style="margin-right: 1em;"><input id="set_fix_x" text="КорректировкаX" type="text" size="3" tooltip="Задайте масштаб изменения размера холста по горизонтали"/></div>',
					'<div class="tbl_opt_label" ><p class="inline">Отступ(слева)</p></div>',
					'<div class="tbl_opt" style="margin-right: 1em;"><input id="set_left_offset" text="Отступ(слева)" type="text" size="3" value="0.0" tooltip="Задайте масштаб изменения размера холста по горизонтали"/></div>',
				'</div>',
			'</div>',
			'<div>',
				'<div class="instrument_line" style="width: 70%; float:left; height: 100%; clear:left">',
					'<div class="tbl_opt_label" ><p class="inline">СжатиеY</p></div>',
					'<div class="tbl_opt" style="margin-right: 1em;"><input id="set_fix_y" text="КорректировкаY" type="text" size="3" tooltip="Задайте масштаб изменения размера холста по вертикали"/></div>',
					'<div class="tbl_opt_label" ><p class="inline">Отступ(сверху)</p></div>',
					'<div class="tbl_opt" style="margin-right: 1em;"><input id="set_top_offset" text="Отступ(слева)" type="text" size="3" value="0.0" tooltip="Задайте масштаб изменения размера холста по горизонтали"/></div>',
				'</div>',
			'</div><div>'
		].join("\r\n");

		var OffsetSetupView = ToolView.extend({
			template: template,
			tagName: 'div',
			events: {
				'change #set_fix_x': 'setScaleX',
				'change #set_fix_y': 'setScaleY',
				'change #set_top_offset': 'setTopOffset',
				'change #set_left_offset': 'setLeftOffset'
			},
			initialize: function(config) {
				this.editor_model = config['editor_model'];
			},
			setScaleX: function() {
				var control = this.$('#set_fix_x');
				var val = control.val();
				if (!this.editor_model.get('offsetMode')) {
					this.pubSub.trigger('scale_x:set', val);
				} else {
					control.val(this.editor_model.get('fix_x'));
				}
			},
			setScaleY: function() {
				var control = this.$('#set_fix_y');
				var val = control.val();
				if (!this.editor_model.get('offsetMode')) {
					this.pubSub.trigger('scale_y:set', val);
				} else {
					control.val(this.editor_model.get('fix_y'));;
				}
			},
			setTopOffset: function() {
				var control = this.$('#set_top_offset');
				var val = control.val();
				if (!this.editor_model.get('offsetMode')) {
					this.pubSub.trigger('top_offset:set', val);
				} else {
					control.val(this.editor_model.get('offset_y'));
				}
			},
			setLeftOffset: function() {
				var val = this.$('#set_left_offset').val();
				if (!this.editor_model.get('offsetMode')) {
					this.pubSub.trigger('left_offset:set', val);
				} else {
					this.$('#set_left_offset').val(this.editor_model.get('offset_x'));
				}
			},
			updateOffsetX: function(model, val) {
				this.$('#set_left_offset').val(val);
			},
			updateOffsetY: function(model, val) {
				this.$('#set_top_offset').val(val);
			},
			updateScaleX: function(model, val) {
				this.$('#set_fix_x').val(val);
			},
			updateScaleY: function(model, val) {
				this.$('#set_fix_y').val(val);
			},
			render: function() {
				this.$el.html(this.template);
				this.$el.css("padding", '1em');
				return this;
			}
		});

		return OffsetSetupView;
	}
);