define('views/toolbox_view', ['jquery', 'backbone'], 
	function($, Backbone) {

        var template = [
            "<div class='akordeon'>",
            "<div id='editor_panel_roll_btn'>",
            "</div>",
            "</div>"
        ].join("\r\n");

        var panelDefaultWidth = 802;

        var ToolboxView = Backbone.View.extend({

            template: template,
            id: 'info',
            tagName: 'div',
            events: {
                'change #extended_tools'      : 'showExtended',
                'click #editor_panel_roll_btn': 'onToolboxShow'
            },

            initialize: function(config) {
                this.editor = config.editor;
            },
            afterRender: function() {
                var workspace, panel = this.$el;
                workspace = this.getWorkspace();

                this.toolboxShowButton = this.$("#editor_panel_roll_btn");
                this.toolboxShowButton.addClass("off");

                panel.css("left",
                    (parseInt(workspace.attr("width")) + 50).toString() + "px");

                $(window).resize(function() {
                    panel.css("marginRight", "-802px");
                });
            },
            render: function() {
                this.$el.html(this.template);
                return this;
            },

            getWorkspace: function() {
                var canvas = this.editor.canvas,
                    canvas_container = this.editor.container, workspace;

                if (parseFloat(canvas_container.css("width")) > parseFloat(canvas.css("width"))) {
                    workspace = canvas_container;
                } else {
                    workspace = canvas;
                }

                return workspace;
            },

            onToolboxShow: function() {

                //обработка события нажатия на кнопку показа/скрытия панели инструментов
                var btn = this.toolboxShowButton,
                    cvas = this.getWorkspace();

                if (!btn.hasClass("blocked") && btn.hasClass("on")) {
                    btn.addClass("blocked");

                    this.$el.animate({"marginRight": "-802px"}, 1000, function(){
                        btn.removeClass("blocked");
                        btn.removeClass("on");
                        btn.addClass("off");
                        btn.css("backgroundImage",
                            staticUrl + "/images/editor/show_edit.png");
                    });
                }
                else if (!btn.hasClass("blocked") && btn.hasClass("off")) {
                    btn.addClass("blocked");

                    var doc_width = $(window).width(),
                        coord_view = this.$el.siblings("ul").find(".table_w"),
                        inst_panel_width = coord_view.length ?
                                 parseFloat(coord_view.css("width")) + 60 : panelDefaultWidth + 60,
                        cvas_container = cvas.parent(),
                        cvas_width = parseFloat(cvas.css("width")),
                        restrict_width = parseFloat(cvas_container.css('width')) + inst_panel_width,
                        set_margin = 0;

                    if (doc_width < restrict_width) {
                        set_margin = (doc_width - inst_panel_width) - cvas_width;
                    }

                    this.$el.animate({"marginRight": set_margin + "px"}, 1000, function(){
                        btn.removeClass("blocked");
                        btn.removeClass("off");
                        btn.addClass("on");
                        btn.css("backgroundImage",
                            staticUrl + "/images/hide_edit.png");
                    });

                }    
            },

            addTool: function(view) {
                //добавление нового представления являющиеся инструментом в терминах редактора
                this.$el.append(view.render().el);
            }
        });

        return ToolboxView;
	}
);