#coding: utf-8

import copy
import json
from django.template.context import RequestContext
import os
import re

from django.http import HttpResponse
from django.conf import settings
from django.shortcuts import render_to_response

from .api import BlankGenerator

def get_user(request):

    class FakeUser(object):
        id = 1
        is_superuser = True

    #шаблоны могут не быть привязанными к пользователю
    try:
        user_is_auth = request.user.is_authenticated()
    except Exception as auth_err:
        user_is_auth = False

    if user_is_auth:
        user = request.user
    else:
        user = FakeUser()

    return user


def test_runner(request):

    cxt = {
        "BLANK_EDITOR_MEDIA_URL": settings.BLANK_EDITOR_MEDIA_URL
    }

    return render_to_response("SpecRunner.html", cxt)


def main(request):

    class FakeUser(object):
        id = 1

    blank1_generator = BlankGenerator("TestProvider1", add_values = {"test": 1})
    blank2_generator = BlankGenerator("TestProvider2")
    user = FakeUser()

    ctx = {
        "TestProvider1handler": blank1_generator.get_print_handler(0, user),
        "TestProvider2handler": blank2_generator.get_print_handler(0, user),
        "TestProvider1title"  : blank1_generator.provider.title,
        "TestProvider2title"  : blank2_generator.provider.title,
        "BLANK_EDITOR_STATIC_URL": settings.BLANK_EDITOR_STATIC_URL
    }

    return render_to_response("editor_test.html",
        ctx, context_instance=RequestContext(request))


def test_report_view(request):

    prs = request.POST
    provider_name = prs.get('pack_url')
    provider = BlankGenerator(provider_name).provider
    params = [
        (1, "last_name",    u"Фамилия"),
        (2, "first_name",   u"Имя"),
        (3, "last_name",    u"Отчество")
    ]
    ctx = [
        [param[1] for param in params],
        [param[2] for param in params]
    ]
    return HttpResponse(content_type="json", content=json.dumps(ctx))


def get_all_templates(request):

    provider_name = request.GET.get('pack_url')
    utils = BlankGenerator(provider_name)
    user = get_user(request)
    saved_blanks = utils.get_saved_blanks_by_user(user)

    return HttpResponse(content=json.dumps(saved_blanks), content_type="json")


def save_template(request):

    prs = request.POST.get
    provider_name, name = map(lambda x: x.strip(), map(prs, ["pack_url", "name"]))
    coord_map = json.loads(prs('coord_map').strip())
    user_map = json.loads(prs('user_map').strip())
    result = {"success" : True}

    blank_generator = BlankGenerator(provider_name)
    raw_path = blank_generator.template_path
    report_path_join = raw_path.split() if "/" in raw_path else raw_path.split("\\")
    #TODO - os.sep неправильно применять
    path_to_report = os.sep.join(report_path_join)

    #преобразуем unicode строки в utf-8
    for k, f in enumerate(coord_map):
        coord_map[k] = map(lambda x: x.encode("utf-8") if isinstance(x, unicode) else x, f)
    for k, f in enumerate(user_map):
        user_map[k] = map(lambda x: x.encode("utf-8") if isinstance(x, unicode) else x, f)

    extend_header = "{% extends \""+blank_generator.default_blank_name+"\" %}\r\n"
    template_block = extend_header + "{% block script %}\r\n"

    blank_variables = """
    coord_map = [
         %(coord_map)s
    ];
    user_map = [
        %(user_map)s
    ];
    background_img_url = '%(background_url)s';
    fix_x = %(fix_x)s;
    fix_y = %(fix_y)s;
    columns_count = %(columns_count)s;
    cell_rng_start = %(cell_rng_start)s;
    cell_rng_end = %(cell_rng_end)s;
    offset_x = %(offset_x)s;
    offset_y = %(offset_y)s;
    hyphenation_on = %(hyphenation_on)s;
    font_type='%(font_type)s';
    font_size='%(font_size)s';
    """ % ({
        "coord_map": ",\r".join([str(r) for r in coord_map]),
        "user_map": ",\r".join([str(r) for r in user_map]),
        "background_url": prs("background_url"),
        "fix_x": prs("fix_x"),
        "fix_y": prs("fix_y"),
        "columns_count": prs("columns_count"),
        "cell_rng_start": prs("cell_rng_start"),
        "cell_rng_end": prs("cell_rng_end"),
        "offset_x": prs("offset_x"),
        "offset_y": prs("offset_y"),
        "hyphenation_on": prs("hyphenation_on"),
        "font_size": prs("font_size"),
        "font_type": prs("font_type")
    })
    html_template = "".join([template_block, blank_variables, "{% endblock %}"])
    #формат сохраненного файла шаблона выглядит следующим образом
    #template_uidXXXX_[name].html где XXXX - это id пользователя в системе
    #а name - это произвольное имя шаблона заданное пользователем
    filt_names = []
    user = get_user(request)

    for dirname, dirnames, filenames in os.walk(path_to_report):
        filt_names = [f.replace(".html",'').replace("template_", '').replace("uid", '')[5:]
                      for f in filenames
                      if "template_uid{0:0>4}_".format(user.id) in f
                      or "template_uid0000_basic_" in f]

    end_template_name = os.path.join(path_to_report,
        "template_uid{0:0>4}_".format(user.id) + name + ".html").replace("\\", "/")

    if user.is_superuser and name == u"basic":
        end_template_name = os.path.join(path_to_report, "template_uid0000_" + name + ".html")

    if not user.is_superuser and name == 'basic':
        result['success'] = False
        result['message'] = u"Базовый шаблон может быть сохранен только администратором системы!"
    elif re.search(ur"[А-Я,а-я]*", name).group():
        result['success'] = False
        result['message'] = u"В имени файла запрещено использовать не латинские символы!"
    elif len(name)>10:
        result['success'] = False
        result['message'] = u"Длина имени файла не должна превышать 10 символов!"
    elif len(filt_names)>=3  and name not in filt_names and name != 'basic':
        result['success'] = False
        result['message'] = u"Вы не можете сохранять больше трех шаблонов!"
    else:
        if user:
            with open(end_template_name, "w") as fh:
                fh.write(html_template.encode("utf-8"))
        else:
            result['success'] = False
    result = json.dumps(result)

    return HttpResponse(content_type="json", content=result)


def print_template(request, context=None):

    prs = request.POST.get
    exist_tempfile = (prs("exist_tempfile", None)
                      or (context and getattr(context, 'exist_tempfile', None)))
    if exist_tempfile:
        exist_tempfile = exist_tempfile.strip()
    provider_name = prs("pack_url").strip()
    report_data = (prs("report_data")
                   or (context and getattr(context , 'report_data', [])))
    data_description = (prs("data_description")
                        or (context and getattr(context, 'data_description', [])))
    gen = BlankGenerator(provider_name)
    provider = gen.provider

    cfg = dict(
        path_template=((prs("path_template") and prs("path_template").strip())
                       or (context and context.get('path_template', None))),
        number_of_params=(prs("number_of_params").strip()
                          or (context and context.get('number_of_params', None))),
        pack_url=prs("pack_url").strip() or (context and context.get('pack_url', '')),
        blank_size=provider.blank_size,
        id_field=prs("id_field").strip() or (context and context.get('id_field', '')),
        report_data=report_data,
        data_description=data_description,
        background_reload_url=(prs("background_reload_url", "")
            or (context and context.get("background_reload_url", "")) or ''),
        add_vals=prs('add_vals')
    )

    res = gen.get_report_response(cfg, request, exist_tempfile)

    return res


def get_template_background(request):

    prs = request.POST.get

    filename = "background_image.jpg"
    (provider_name, template_name,
     template_path) = map(lambda x: x.strip() if x else '',
                      map(prs, ['pack_url', 'template_name', 'template_path']))
    template_name = request.GET.get("template_name", '')

    generator = BlankGenerator(provider_name)
    context = {}
    #получаем путь для хранения файла изображения
    path_to_background = generator.media_path
    static_path_to_file = path_to_background.split(settings.BLANK_EDITOR_MEDIA_ROOT.replace('\\', '/'))[1][1:] + "/"

    user = get_user(request)
    if template_name:
        context['path_template'] = "/".join(template_path.split("/")[:-1])
        if template_name != "basic":
            context['path_template'] += "/template_uid{0:0>4}_".format(user.id)
        else:
            context['path_template'] += "/template_uid0000_"
        context['path_template'] += template_name + ".html"
    else:
        context['path_template'] = template_path

    if request.FILES:
        with open(os.path.join(path_to_background, filename), "wb") as f:
            info = request.FILES['select_file'].file.read()
            f.write(info)
            context['old_file'] = f.name

    context['report_data'] = []
    context['data_description'] = []
    context['report_pack'] = provider_name
    context['background_reload_url'] = static_path_to_file + filename

    #вызов функции печати на бланке
    res = print_template(request, context)

    return res


def get_data_for_report(request):

    prs = request.POST.get
    id_field, provider_name, add_vals = map(prs, ['id_field', 'pack_url', 'add_vals'])
    user = get_user(request)
    provider = BlankGenerator(provider_name).provider
    territory = None
    data = []
    kwargs = copy.deepcopy(request.POST)
    delete_params = ['pack_url', 'id_field', 'add_vals']
    for dp in delete_params:
        if dp in kwargs:
            del kwargs[dp]

    if add_vals:
        add_vals = json.loads(add_vals)

    if add_vals and isinstance(add_vals, dict):
        for e, v in add_vals.items():
            kwargs[e.encode('utf-8')] = v

    kwargs['user'] = user
    #возвращаем данные для отчета по идентификатору записи в журнале
    data_values = provider.getDataForReport(provider.__class__,
        id_field, territory=territory, type_="values", **kwargs)

    data_values = [v.replace(u'\\', u'\\\\')
                   if isinstance(v, unicode)
                   or isinstance(v, str) else v for v in data_values]

    data.append(data_values)
    data.append(provider.getDataForReport(provider.__class__,
        id_field, territory=territory, type_="descriptions"))

    return HttpResponse(content=json.dumps(data), content_type="json")
